import os
import time
import json
import numpy as np
from pychemia.crystal import KPoints
from pychemia import pcm_log
from pychemia.utils.serializer import generic_serializer
from ..vasp import VaspJob
from ..outcar import read_vasp_stdout
from ...tasks import Task

__author__ = 'Guillermo Avendano-Franco'


class StaticCalculation(Task):
    def __init__(self, structure, workdir='.', binary='vasp', encut=1.3, kpoints=None, kp_density=1E4,
                 extra_incar=None):

        self.encut = encut
        if kpoints is None:
            kp = KPoints.optimized_grid(structure.lattice, kp_density=kp_density, force_odd=True)
            self.kpoints = kp
        else:
            self.kpoints = kpoints
        self.task_params = {'encut': self.encut, 'kpoints': self.kpoints.to_dict, 'extra_incar': extra_incar}
        Task.__init__(self, structure=structure, task_params=self.task_params, workdir=workdir, binary=binary)

    def run(self, nparal=4):

        vj = VaspJob()
        vj.initialize(self.structure, self.workdir, self.kpoints, binary=self.binary)
        vj.clean()
        vj.job_static()
        vj.input_variables.set_density_for_restart()
        vj.input_variables.set_encut(ENCUT=self.encut, POTCAR=self.workdir + os.sep + 'POTCAR')
        vj.input_variables.variables['NBANDS'] = nparal * ((int(self.structure.valence_electrons()) +
                                                            self.structure.natom) / nparal + 1)
        vj.input_variables.set_ismear(self.kpoints)
        vj.input_variables.variables['SIGMA'] = 0.2
        vj.input_variables.variables['ISPIN'] = 2
        if self.task_params['extra_incar'] is not None:
            for i in self.task_params['extra_incar']:
                vj.input_variables.variables[i] = self.task_params['extra_incar'][i]
        vj.set_inputs()
        self.encut = vj.input_variables.variables['ENCUT']
        vj.run(use_mpi=True, mpi_num_procs=nparal)
        pcm_log.debug('Starting VASP')
        while True:
            energy_str = ''
            filename = self.workdir + os.sep + 'vasp_stdout.log'
            if os.path.exists(filename):
                vasp_stdout = read_vasp_stdout(filename=filename)
                if len(vasp_stdout['data']) > 2:
                    scf_energies = [i[2] for i in vasp_stdout['data']]
                    energy_str = ' %7.3f' % scf_energies[1]
                    for i in range(1, len(scf_energies)):
                        if scf_energies[i] < scf_energies[i - 1]:
                            energy_str += ' >'
                        else:
                            energy_str += ' <'
                    pcm_log.debug(energy_str)

            if vj.runner is not None and vj.runner.poll() is not None:
                filename = self.workdir + os.sep + 'vasp_stdout.log'
                if os.path.exists(filename):
                    vasp_stdout = read_vasp_stdout(filename=filename)
                    if len(vasp_stdout['data']) > 2:
                        scf_energies = [i[2] for i in vasp_stdout['data']]
                        energy_str += ' %7.3f' % scf_energies[-1]
                        pcm_log.debug(energy_str)
                pcm_log.debug('Execution complete')
                break
            time.sleep(5)
        vj.get_outputs()

        self.output = {'forces': generic_serializer(vj.outcar.forces), 'stress': generic_serializer(vj.outcar.stress),
                       'energy': vj.outcar.energy, 'energies': generic_serializer(vj.outcar.energies),
                       'INCAR': vj.input_variables.variables}
        if vj.outcar.is_finished:
            self.finished = True

    def plot(self, figname='static_calculation.pdf'):
        if not self.finished:
            print('The task is not finished')
            return
        import matplotlib.pyplot as plt
        plt.switch_backend('agg')
        plt.figure(figsize=(8, 6))
        plt.subplots_adjust(left=0.09, bottom=0.08, right=0.95, top=0.95, wspace=None, hspace=None)
        data = np.array(self.output['energies'])
        plt.plot(data[:, 1], data[:, 2], 'b.-')
        plt.xlabel('SCF cycle')
        plt.ylabel('Energy [eV]')

        a = plt.axes([.6, .6, .3, .3], axisbg='0.9')
        a.semilogy(data[:, 1], data[:, 2] - np.min(data[:, 2]))
        a.set_title('min energy %7.3f eV' % np.min(data[:, 2]))

        if figname is not None:
            plt.savefig(figname)
        return plt.gcf()

    def load(self, filename=None):
        if filename is None:
            filename = self.workdir + os.sep + 'task.json'
        rf = open(filename)
        data = json.load(rf)
        rf.close()
        self.task_params = data['task_params']
        self.output = data['output']
        self.encut = self.task_params['encut']
        self.kpoints = KPoints.from_dict(self.task_params['kpoints'])

    def report(self, file_format='html'):
        from lxml.builder import ElementMaker, E
        self.plot(figname=self.report_dir + os.sep + 'static.jpg')

        element_maker = ElementMaker(namespace=None, nsmap={None: "http://www.w3.org/1999/xhtml"})
        html = element_maker.html(E.head(E.title("VASP Static Calculation")),
                                  E.body(E.h1("VASP Static Calculation"),
                                         E.h2('Structure'),
                                         E.pre(str(self.structure)),
                                         E.h2('Self Consistent Field Convergence'),
                                         E.p(E.img(src='static.jpg', width="800", height="600",
                                                   alt="Static Calculation"))
                                         ))

        return self.report_end(html, file_format)



from __future__ import print_function
import numpy as np
import pychemia
import itertools
import scipy.spatial
from scipy.spatial import qhull
from pychemia.utils.periodic import covalent_radius


# return [x, y, d], that ax + by = d, d = gcd(a, b)
def ext_gcd(a, b):
    v1 = [1, 0, a]
    v2 = [0, 1, b]

    if a > b:
        a, b = b, a
    while v1[2] > 0:
        q = v2[2] / v1[2]
        for i in range(0, len(v1)):
            v2[i] -= q * v1[i]
        v1, v2 = v2, v1
    return v2


def print_vector(v):
    print('v = ', end='')
    for i in range(3):
        print(v[i], end='')
    print("\n")


def rotate_along_indices(structure, h, k, l, layers, tol=1.e-5):
    cell = structure.cell
    a1 = np.array(cell[0])
    a2 = np.array(cell[1])
    a3 = np.array(cell[2])
    rcell = structure.lattice.reciprocal().cell
    b1 = np.array(rcell[0])
    b2 = np.array(rcell[1])
    b3 = np.array(rcell[2])

    # Solve equation pk + ql = 1 for p and q using extended_Euclidean algorithm

    v = ext_gcd(k, l)
    p = v[0]
    q = v[1]
    # print('p = ', p)
    # print('q = ', q)

    k1 = np.dot(p * (k * a1 - h * a2) + q * (l * a1 - h * a3), l * a2 - k * a3)
    k2 = np.dot(l * (k * a1 - h * a2) - k * (l * a1 - h * a3), l * a2 - k * a3)
    # print("\n\nk1 = ", k1)
    # print("k2 = ", k2)

    if abs(k2) > tol:
        c = -int(round(k1 / k2))
        # print("c = -int(round(k1/k2)) = ", c)
        p, q = p + c * l, q - c * k

    # Calculate lattice vectors {v1, v2, v3} defining basis of the new cell

    v1 = p * (k * a1 - h * a2) + q * (l * a1 - h * a3)
    v2 = l * a2 - k * a3
    n = p * k + q * l
    v = ext_gcd(n, h)
    a = v[0]
    b = v[1]
    v3 = b * a1 + a * p * a2 + a * q * a3
    newbasis = np.array([v1, v2, v3])

    #    transformation = np.array([[p*k+q*l, -h*p, -h], [0, l, -k], [b, a*p, a*q]])
    #    inv_transformation = np.linalg.inv(transformation)

    symbols = []
    positions = structure.positions + np.tile(np.dot(structure.cell.T, np.array([0, 0, 0])).reshape(1, 3),
                                              (structure.natom, 1))
    reduced = np.linalg.solve(newbasis.T, positions.T).T
    for i in range(3):
        reduced[:, i] %= 1.0
    symbols += structure.symbols
    surf = pychemia.Structure(symbols=symbols, reduced=reduced, cell=newbasis)

    if layers > 1:
        new_surf = surf.supercell((1, 1, layers))
        cell = new_surf.cell
        # cell[2] = cell[2]+(layers+1)*cell[1]+(layers+1)*cell[0]
        surf = pychemia.Structure(symbols=new_surf.symbols, positions=new_surf.positions, cell=cell)

    #    print '\n\n********* Lattice vectors of the original cell *********\n\n', cell
    #    print '\n\n********* ATOMIC positions in the original cell **********\n', structure.positions
    #    print '\nTotal number of atoms in cell = ', structure.natom

    # Now create the surface starting from the original structure
    #    surf = structure.copy()
    #    surf.set_cell(newbasis)

    #    print '\n\n********* New basis of the surface cell *********\n\n', surf.cell
    #    print '\n\n********* Atomic coordinates in the newbasis of surface cell *********\n\n', surf.positions

    #    surf = surf.supercell((1, 1, layers))
    #    a1, a2, a3 = surf.cell

    #    surf.set_cell([a1, a2,
    #                   np.cross(a1, a2) * np.dot(a3, np.cross(a1, a2)) /
    #                   np.linalg.norm(np.cross(a1, a2)) ** 2])

    # Change unit cell to have the x-axis parallel with a surface vector
    # and z perpendicular to the surface:

    #    a1, a2, a3 = surf.cell
    #    surf.set_cell([(np.linalg.norm(a1), 0, 0),
    #                   (np.dot(a1, a2) / np.linalg.norm(a1),
    #                    np.sqrt(np.linalg.norm(a2) ** 2 - (np.dot(a1, a2) / np.linalg.norm(a1)) ** 2), 0),
    #                   (0, 0, np.linalg.norm(a3))])

    # Move atoms into the unit cell:

    #    scaled = surf.reduced
    #    scaled[:, :2] %= 1
    #    surf.set_reduced(scaled)
    #    surf.center(vacuum=vacuum, axis=2)
    return surf


def get_surface_atoms(structure):
    """
    Returns the list of atoms that belong to the surface of a given structure
    The surface atoms are computed as those atom for which the Voronoi tesselation is
    open, ie, their voronoi volume associated is infinite.

    :param structure: PyChemia Structure (Non-periodic in the current implementation)
    :return: (list) List of integers with the indices of the surface atoms
    """
    assert (not structure.is_periodic)
    voro = scipy.spatial.Voronoi(structure.positions)
    surface = [i for i in range(structure.natom) if -1 in voro.regions[voro.point_region[i]]]
    return surface


def get_surface_atoms_new(structure, use_covalent_radius=False):
    dln = scipy.spatial.Delaunay(structure.positions)

    if use_covalent_radius:
        simplices = []
        for j in dln.simplices:
            discard = False
            for ifacet in list(itertools.combinations(j, 3)):
                for ipair in itertools.combinations(ifacet, 2):
                    distance = np.linalg.norm(structure.positions[ipair[0]] - structure.positions[ipair[1]])
                    cov_distance = covalent_radius(structure.symbols[ipair[0]]) + covalent_radius(
                        structure.symbols[ipair[1]])
                    if distance > 3.0*cov_distance:
                        print('Distance: %f Cov-distance: %f' % (distance, cov_distance))
                        discard = True
                        break
            if not discard:
                print(j)
                simplices.append(j)
    else:
        simplices = dln.simplices

    c = np.array([[sorted(list(y)) for y in (itertools.combinations(x, 3))] for x in simplices])
    d = [list(x) for x in c.reshape((-1, 3))]

    ret = []
    dups = []
    for i in range(len(d) - 1):
        if d[i] in d[i+1:]:
            dups.append(d[i])
    for i in d:
        if i not in dups:
            ret.append(i)
    return np.unique(np.array(ret).flatten())


def get_onion_layers(structure):
    """
    Returns the different layers of a finite structure

    :param structure:
    :return:
    """
    assert (not structure.is_periodic)
    layers = []
    cur_st = structure.copy()

    morelayers = True
    while morelayers:
        pos = cur_st.positions
        if len(pos) <= 4:
            core = range(cur_st.natom)
            layers.append(core)
            break

        st = pychemia.Structure(positions=pos, symbols=len(pos)*['H'], periodicity=False)
        st.canonical_form()
        # print('The current volume is %7.3f' % st.volume)
        if st.volume < 0.1:
            core = range(cur_st.natom)
            layers.append(core)
            break

        try:
            voro = scipy.spatial.Voronoi(pos)
            surface = [i for i in range(cur_st.natom) if -1 in voro.regions[voro.point_region[i]]]
        except qhull.QhullError:
            surface = range(cur_st.natom)
            morelayers = False
        layers.append(surface)
        if not morelayers:
            break
        core = [i for i in range(cur_st.natom) if i not in surface]
        if len(core) == 0:
            break
        symbols = list(np.array(cur_st.symbols)[np.array(core)])
        positions = cur_st.positions[np.array(core)]
        cur_st = pychemia.Structure(symbols=symbols, positions=positions, periodicity=False)
    new_layers = [layers[0]]
    included = list(layers[0])
    acumulator = 0
    for i in range(1, len(layers)):
        noincluded = [j for j in range(structure.natom) if j not in included]
        # print 'Layer: %3d   Atoms on Surface: %3d     Internal Atoms: %3d' % (i,
        #                                                                      len(included) - acumulator,
        #                                                                      len(noincluded))
        acumulator = len(included)
        relabel = [noincluded[j] for j in layers[i]]
        included += relabel
        new_layers.append(relabel)

    return new_layers


def get_facets(structure, surface, seed, distance_tolerance=2.0):
    if seed not in surface:
        print('Error: seed not in surface')
        print('Seed: ', seed)
        print('Surface: ', surface)
        raise ValueError('Seed not in surface')

    idx_seed = surface.index(seed)
    pos = structure.positions[surface]
    dl = scipy.spatial.Delaunay(pos)

    # Delaunay tetrahedrals
    # The indices for each tetragon are relative to the surface
    # not the structure
    tetra = [x for x in dl.simplices if idx_seed in x]

    # Determining Facets
    facets = []
    for x in tetra:
        facets += [y for y in list(itertools.combinations(x, 3)) if idx_seed in y]

    # Indices of facets relative to surface not the structure
    facets = list(tuple([sorted(x) for x in facets]))

    selected_facets = []
    mintol = 1E5
    for x in facets:
        dm = scipy.spatial.distance_matrix(pos[x], pos[x])
        maxdist = max(dm.flatten())
        pair = np.where(dm == maxdist)
        atom1 = x[pair[0][0]]
        atom2 = x[pair[1][0]]
        covrad1 = pychemia.utils.periodic.covalent_radius(structure.symbols[surface[atom1]])
        covrad2 = pychemia.utils.periodic.covalent_radius(structure.symbols[surface[atom2]])
        if maxdist / (covrad1 + covrad2) < mintol:
            mintol = maxdist / (covrad1 + covrad2)
        if maxdist < distance_tolerance * (covrad1 + covrad2):
            # Converting indices from surface to structure
            st_facet = tuple([surface[x[i]] for i in range(3)])
            selected_facets.append(st_facet)
    return selected_facets, mintol + 0.1


def get_center_vector(structure, facet):
    v0 = np.array(structure.positions[facet[0]])
    v1 = np.array(structure.positions[facet[1]])
    v2 = np.array(structure.positions[facet[2]])
    facet_center = 1.0 / 3.0 * (v0 + v1 + v2)
    facet_vector = pychemia.utils.mathematics.unit_vector(np.cross(v1 - v0, v2 - v0))
    center_mass = structure.center_mass()
    if np.dot(facet_center - center_mass, facet_vector) < 0:
        facet_vector *= -1
    return (v0, v1, v2), facet_center, facet_vector


def attach_to_facet(structure, facet):
    (v0, v1, v2), facet_center, facet_vector = get_center_vector(structure, facet)

    rnd = np.random.random()
    if rnd < 0.4:
        return facet_center, facet_vector, tuple(facet)
    elif rnd < 0.6:
        edge_center = 0.5 * (v0 + v1)
        return edge_center, facet_vector, (facet[0], facet[1])
    elif rnd < 0.8:
        edge_center = 0.5 * (v1 + v2)
        return edge_center, facet_vector, (facet[1], facet[2])
    else:
        edge_center = 0.5 * (v0 + v2)
        return edge_center, facet_vector, (facet[0], facet[2])


def random_attaching(structure, seed, target_species, natom_crystal, radius=1.8, basetol=4.0):

    lys = pychemia.analysis.surface.get_onion_layers(structure)
    surface = lys[0]

    counter = 0
    while True:
        if seed not in surface or counter > 0:
            print('Current Seed not in surface, searching a new seed')
            seed = find_new_seed(structure, surface, seed, natom_crystal)

        tol = basetol
        facets = []
        while True:
            facets, mintol = get_facets(structure, surface, seed, distance_tolerance=tol)
            if len(facets) > 0:
                print('Possible Facets', facets)
                break
            elif mintol > 2 * basetol:
                return None, None, None, None
            else:
                tol = mintol
                print('No facets found, increasing tolerance to ', tol)

        counter = 0
        while True:
            counter += 1
            rnd = np.random.randint(len(facets))
            facet_chosen = facets[rnd]
            print('Seed: %3d     Number of facets: %3d     Facet chosen: %s' % (seed, len(facets), facet_chosen))

            center, uvector, atoms_facet = attach_to_facet(structure, facet_chosen)

            new_sts = {}
            good_pos = 0
            for specie in target_species:
                cov_rad = pychemia.utils.periodic.covalent_radius(specie)
                vec = center + radius * cov_rad * uvector
                new_symbols = list(structure.symbols) + [specie]
                new_positions = np.concatenate((structure.positions, [vec]))

                dist_matrix = scipy.spatial.distance_matrix(new_positions, new_positions)
                identity = np.eye(len(new_positions), len(new_positions))
                mindist = np.min((dist_matrix + 100 * identity).flatten())
                if mindist > cov_rad:
                    good_pos += 1
                    print('We have a minimal distance of', mindist)
                new_sts[specie] = pychemia.Structure(symbols=new_symbols, positions=new_positions, periodicity=False)

            if good_pos == len(target_species):
                print('Good position selected for all species')
                break
            else:
                print('No enough good positions: %d. One bad position, choosing a new facet' % good_pos)
                if counter > len(facets):
                    break
        if good_pos == len(target_species):
            break
    return new_sts, facet_chosen, center, uvector


def find_new_seed(st, surface, seed, natom_crystal):
    """
    Find a new atom to serve as seed for deposition

    :param st: Structure
    :param surface: Indices of atoms on surface
    :param seed: Current seed
    :param natom_crystal: Number of atoms in crystal, cluster atoms are always at the end
    :return:
    """

    candidates = [x for x in surface if x > natom_crystal]
    if seed in surface:
        new_seed = seed
    else:
        dists = scipy.spatial.distance_matrix(st.positions[seed].reshape((1, 3)),
                                              st.positions[surface])

        # First Option
        new_seed = surface[dists[0].argsort()[0]]

        # Second option
        dists = scipy.spatial.distance_matrix(st.center_mass().reshape((1, 3)),
                                              st.positions[candidates])

    if len(candidates) > 0:
        dists = scipy.spatial.distance_matrix(st.center_mass().reshape((1, 3)), st.positions[candidates])
        npcandidates = np.array(candidates)[dists[0].argsort()]
        print('Candidates ordered by distance to CM', npcandidates)
        for i in npcandidates:
            facets, mintol = get_facets(st, surface, i, distance_tolerance=1.0)
            if mintol < 5.0:
                new_seed = i
                break

    print('Seed: %3d => %3d' % (seed, new_seed))
    assert (new_seed in surface)
    return int(new_seed)



import pychemia


def test_abinit_parser():
    """
    Tests (pychemia.code.abinit) [parser]                        :
    """
    if pychemia.HAS_SCIPY and pychemia.HAS_SCIENTIFIC:
        from pychemia.code.abinit import parser
        from numpy import array, all, ones
        from math import sqrt
        import tempfile

        wf = tempfile.NamedTemporaryFile(mode='w')
        wf.write('    # Comentario\n')
        wf.write('    ! Comentario\n')
        wf.write('\n')
        wf.write('inputvar1 1 # integer\n')
        wf.write('inputvar2 1.2 # float\n')
        wf.write('inputvar3 3*4 # list of integer\n')
        wf.write('inputvar4 3*4.5 # list of float\n')
        wf.write('inputvar5 3*4.5e6 # list of float\n')
        wf.write('inputvar6 3*4.5d7 # list of float\n')
        wf.write('inputvar7 3*4.5E6 # list of float\n')
        wf.write('inputvar8 3*4.5D7 # list of float\n')
        wf.write('inputvar9 *1\n')
        wf.write('inputvar10 sqrt(2)\n')
        wf.write('inputvar11 6*sqrt(3)\n')
        wf.flush()

        inp = parser(wf.name)
        wf.close()

        assert len(inp.keys()) == 11
        assert inp['inputvar1'] == array([1])
        assert inp['inputvar2'] == array([1.2])
        assert all(inp['inputvar3'] == 4 * ones(3))
        assert all(inp['inputvar4'] == 4.5 * ones(3))
        assert all(inp['inputvar5'] == 4.5e6 * ones(3))
        assert all(inp['inputvar6'] == 4.5e7 * ones(3))
        assert all(inp['inputvar7'] == 4.5e6 * ones(3))
        assert all(inp['inputvar8'] == 4.5e7 * ones(3))
        assert inp['inputvar9'] == '*1'
        assert inp['inputvar10'] == sqrt(2)
        assert all(inp['inputvar11'] == sqrt(3) * ones(6))


def test_abinit_utils():
    """
    Tests (pychemia.code.abinit) [utils]                         :
    """
    if pychemia.HAS_SCIPY and pychemia.HAS_SCIENTIFIC:
        from pychemia.code.abinit import xyz2input, netcdf2dict, psp_name

        filename = "pychemia/test/data/abinit_05/abinit-o_OUT.nc"
        print(len(netcdf2dict(filename)))
        assert len(netcdf2dict(filename)) == 310
        assert psp_name(1, 'LDA', 'FHI') == '01-H.LDA.fhi'
        filename = "pychemia/test/data/abinit_05/abinit_DS11.xyz"
        assert xyz2input(filename).variables['natom'] == 2


def test_abinit_abifiles():
    """
    Tests (pychemia.code.abinit) [abifiles]                      :
    """
    if pychemia.HAS_SCIPY and pychemia.HAS_SCIENTIFIC:
        from pychemia.code.abinit import AbiFiles

        filename = "pychemia/test/data/abinit_01/abinit.files"
        abf = AbiFiles(filename)
        assert abf.filename == "abinit.files"
        assert abf.get_input_filename() == 'pychemia/test/data/abinit_01/abinit.in'


def test_abinit_input():
    """
    Tests (pychemia.code.abinit) [input]                         :
    """
    if pychemia.HAS_SCIPY and pychemia.HAS_SCIENTIFIC:
        from pychemia.code.abinit import AbiFiles, InputVariables

        filename = "pychemia/test/data/abinit_01/abinit.files"
        abf = AbiFiles(filename)
        inp = InputVariables(abf)
        print(inp)
        print(len(inp))
        assert len(inp) == 31
        assert inp.get_value('ecut') == 10
        assert len(inp.get_dtsets_keys()) == 12
        assert inp.get_value('ntime', 41) == 10
        assert inp.get_value('acell', 41)[0] == 14


def test_abinit():
    """
    Tests (pychemia.code.abinit) [general]                       :
    """
    import pychemia
    if pychemia.HAS_SCIPY and pychemia.HAS_SCIENTIFIC:
        import pychemia.code.abinit

        af = pychemia.code.abinit.AbiFiles(basedir='pychemia/test/data/abinit_03')
        iv = pychemia.code.abinit.InputVariables('pychemia/test/data/abinit_03/rnpg.in')
        af.set_input(iv)
        af.set_psps('LDA', 'FHI')
        af.create()
        iv.write(af.get_input_filename())
        assert len(open(af.get_input_filename()).readlines()) == 71



"""
Set of classes and functions to manipulate

ABINIT '.in' input files
ABINIT '.files' files
ABINIT '_OUT.nc' output files

"""
from pychemia import HAS_SCIPY
from . import task

if HAS_SCIPY:
    from .abifiles import AbiFiles
    from .input import InputVariables, xyz2input
    from .abinit import AbinitJob
    from .output import AbinitOutput, netcdf2dict
    from .utils import psp_name
    from .parser import parser

# __all__ = filter(lambda s: not s.startswith('_'), dir())



import os
import numpy as np
from ._population import Population
from pychemia.utils.mathematics import spherical_to_cartesian, cartesian_to_spherical, rotate_towards_axis, \
    angle_between_vectors
from pychemia.code.vasp import read_incar, read_poscar, VaspOutput


class NonCollinearMagMoms(Population):

    def __init__(self, name, source_dir='.', mag_atoms=None, magmom_magnitude=2.0, distance_tolerance=0.1,
                 incar_extra=None, debug=False):
        """
        This class provides a population of Magnetic Moment vectors for the same structure and was created to be used
        on VASP.
        The magnetic moment is set in cartesian coordinates with 3 numbers for each atom in the unit cell.
        This population provides methods to manipulate the magnetic moments between different candidates in order to
        optimize the magnetic orientations using the global-search methods implemented on PyChemia.

        :param name: The name of the database to be created or directly the PyChemiaDB database object.
                     The name is used when the database can be created without username, password and no encryption.
                     Otherwise the database must be created first and its object be 'name' argument.
        :param source_dir: Directory that contains the basic 4 files for VASP: 'POSCAR', 'POTCAR', 'KPOINTS' and 'INCAR'
                            Except for 'INCAR' the files are linked symbolically on each directory that will run VASP
                            The input variables on 'INCAR' changing only MAGMOM and I_CONSTRAINED_M.
                            The 'INCAR' file could contain generic variables and some other variables could be directly
                            specified using the dictionary 'incar_extra'.
        :param mag_atoms: List of atoms for which the Magnetic Moments are changed. If the variable is None, the list
                            is inferred from the original INCAR file. The numbering of atoms start with 0.
        :param magmom_magnitude: Fix value for the magnitude of the magnetic moment imposed for all the atoms in
                                'mag_atoms' list
        :param distance_tolerance: Maximal distance in magnetic moments to consider two candidates as equivalent.
        :param incar_extra: Extra variables for INCAR file that are added or replaced from the INCAR read with
                            'source_dir'
        :param debug: If True produce a verbose output during the different calls to the methods.
        """
        Population.__init__(self, name, 'global')
        if not os.path.isfile(source_dir + os.sep + 'INCAR'):
            raise ValueError("INCAR not found")
        if not os.path.isfile(source_dir + os.sep + 'POSCAR'):
            raise ValueError("POSCAR not found")
        if not os.path.isfile(source_dir + os.sep + 'POTCAR'):
            raise ValueError("POTCAR not found")
        if not os.path.isfile(source_dir + os.sep + 'KPOINTS'):
            raise ValueError("KPOINTS not found")
        self.input = read_incar(source_dir + os.sep + 'INCAR')
        self.source_dir = source_dir

        if 'MAGMOM' not in self.input:
            raise ValueError('INCAR should define the MAGMOM variable')
        magmom = np.array(self.input.MAGMOM).reshape((-1, 3))

        self.structure = read_poscar(source_dir + os.sep + 'POSCAR')
        if mag_atoms is None:
            self.mag_atoms = list(np.where(np.apply_along_axis(np.linalg.norm, 1, magmom) > 0.0)[0])
            self.mag_atoms = [int(x) for x in self.mag_atoms]
        else:
            self.mag_atoms = mag_atoms
        self.magmom_magnitude = magmom_magnitude
        self.distance_tolerance = distance_tolerance
        if incar_extra is None:
            self.incar_extra = {'IBRION': -1,
                                'LWAVE': True,
                                'LAMBDA': 10,
                                'NSW': 0,
                                'I_CONSTRAINED_M': 1}

        else:
            self.incar_extra = incar_extra
        self.debug = debug

    def __str__(self):
        ret = ' Population NonColl\n\n'
        ret += ' Name:               %s\n' % self.name
        ret += ' Tag:                %s\n' % self.tag
        ret += ' Formula:            %s\n' % self.structure.formula
        ret += ' source_dir:         %s\n' % self.source_dir
        ret += ' magmom_magnitude:   %s\n' % self.magmom_magnitude
        ret += ' distance_tolerance: %s\n' % self.distance_tolerance
        ret += ' incar_extra:        %s\n' % self.incar_extra

        ret += ' Members:            %d\n' % len(self.members)
        ret += ' Actives:            %d\n' % len(self.actives)
        ret += ' Evaluated:          %d\n' % len(self.evaluated)
        return ret

    @property
    def to_dict(self):
        return {'name': self.name,
                'tag': self.tag,
                'mag_atoms': self.mag_atoms,
                'source_dir': self.source_dir,
                'incar_extra': self.incar_extra,
                'magmom_magnitude': self.magmom_magnitude,
                'distance_tolerance': self.distance_tolerance}

    @staticmethod
    def from_dict(self, population_dict):
        return NonCollinearMagMoms(name=population_dict['name'],
                                   source_dir=population_dict['source_dir'],
                                   mag_atoms=population_dict['mag_atoms'],
                                   incar_extra=population_dict['incar_extra'],
                                   magmom_magnitude=population_dict['magmom_magnitude'],
                                   distance_tolerance=population_dict['distance_tolerance'])

    def debug_evaluation(self, magmom_sph):
        """
        For debugging ONLY:
        Fake evaluation of total energy using a magnetic configuration as the minimal
        energy, the numerical distance with other candidates defines their energy.

        :param magmom_sph:
        :return:
        """
        if self.debug:
            magmom_car = spherical_to_cartesian(magmom_sph)
            good_magmom = np.zeros((self.structure.natom, 3))
            for i in self.mag_atoms:
                good_magmom[i] = (-1**i) * 1.15470054 * np.ones(3)
            distance = np.sum(angle_between_vectors(magmom_car, good_magmom))
            distance /= len(self.mag_atoms)
            return distance-np.pi
        else:
            return None

    def evaluate_entry(self, entry_id):
        pass

    def new_entry(self, cartesian_magmoms, active=True):
        """
        Creates a new entry on the database

        :param cartesian_magmoms: Magnetic moments stored in spherical coordinates
        :param active:
        :return:
        """
        cartesian_magmoms = np.array(cartesian_magmoms)
        properties = {'magmom': list(cartesian_magmoms.flatten()), 'energy': self.debug_evaluation(cartesian_magmoms)}
        status = {self.tag: active}
        entry = {'structure': self.structure.to_dict, 'properties': properties, 'status': status}
        entry_id = self.insert_entry(entry)
        return entry_id

    def is_evaluated(self, entry_id):
        """
        One candidate is considered evaluated if it contains any finite value of energy on the properties.energy field

        :param entry_id:
        :return:
        """
        entry = self.get_entry(entry_id, {'_id': 0, 'properties': 1})
        if 'energy' in entry['properties'] and entry['properties']['energy'] is not None:
            return True
        else:
            return False

    def check_duplicates(self, ids):
        """
        Returns a dictionary of non-equivalent candidates as keys and associated to each key
        the list of candidates whose distance is less that the 'distance_tolerance' argument.

        :param ids:
        :return:
        """
        selection = self.ids_sorted(ids)
        ret = {}
        for i in range(len(ids) - 1):
            for j in range(i+1, len(ids)):
                distance = self.distance(selection[i], selection[j])
                if distance < self.distance_tolerance:
                    ret[selection[j]] = selection[i]
        return ret

    def distance(self, entry_id, entry_jd):
        """
        Compute the distance between 2 candidates as the average angular movement
        between the magnetic moments for all the atoms considered magnetic ('mag_atoms')

        :param entry_id:
        :param entry_jd:
        :return:
        """
        entry = self.get_entry(entry_id, {'properties.magmom': 1})
        magmom_i = np.array(entry['properties']['magmom']).reshape((-1, 3))
        entry = self.get_entry(entry_jd, {'properties.magmom': 1})
        magmom_j = np.array(entry['properties']['magmom']).reshape((-1, 3))

        magmom_ixyz = spherical_to_cartesian(magmom_i)
        magmom_jxyz = spherical_to_cartesian(magmom_j)
        distance = np.sum(angle_between_vectors(magmom_ixyz, magmom_jxyz))
        distance /= len(self.mag_atoms)
        return distance

    def move_random(self, entry_id, factor=0.2, in_place=False, kind='move'):
        """
        Change the magnetic orientation randomly for all the atoms considered magnetic 'mag_atoms'
        The 'factor' argument scales the intensite of the movement.

        :param entry_id:
        :param factor:
        :param in_place:
        :param kind:
        :return:
        """
        entry = self.get_entry(entry_id, {'properties.magmom': 1})
        # Magnetic Momenta are stored in spherical coordinates
        magmom_xyz = spherical_to_cartesian(entry['properties']['magmom'])
        # Randomly disturbed using the factor
        magmom_xyz += factor * np.random.random((self.structure.natom, 3)) - factor / 2
        # Reconverting to spherical coordinates
        magmom_new = cartesian_to_spherical(magmom_xyz)
        # Resetting magnitudes
        for i in range(len(magmom_xyz)):
            if magmom_xyz[i][0] > 0.0:
                magmom_new[i, 0] = self.magmom_magnitude
            else:
                magmom_new[i, 0] = 0.0

        properties = {'magmom': list(magmom_new.flatten()), 'energy': self.debug_evaluation(magmom_new)}

        if in_place:
            return self.update_properties(entry_id, new_properties=properties)
        else:
            return self.new_entry(magmom_new, active=False)

    def move(self, entry_id, entry_jd, factor=0.2, in_place=False):
        """
        Move the magnetic moments from one candidate in the direction of another.

        :param entry_id: Source candidate, this one will be moved
        :param entry_jd: Destination candidate, this one is not moved.
        :param factor: (float) a value in range [0,1] 0 being the source, 1 the destination
        :param in_place: (bool) if true the new magnetic moments replace those in entry_id
        :return:
        """
        magmom_new_xyz = np.zeros((self.structure.natom, 3))
        entry = self.get_entry(entry_id, {'properties.magmom': 1})
        magmom_i = np.array(entry['properties']['magmom']).reshape((-1, 3))
        magmom_ixyz = spherical_to_cartesian(magmom_i)
        entry = self.get_entry(entry_jd, {'properties.magmom': 1})
        magmom_j = np.array(entry['properties']['magmom']).reshape((-1, 3))
        magmom_jxyz = spherical_to_cartesian(magmom_j)

        for i in self.mag_atoms:
            # print('Atom %d' % i)
            if magmom_i[i][0] > 0 and magmom_j[i][0] > 0:
                magmom_new_xyz[i] = rotate_towards_axis(magmom_ixyz[i], magmom_jxyz[i], fraction=factor)
                # print('Final magmom Cartesian : %d     %s' % (i,magmom_new_xyz[i]))

        magmom_new = cartesian_to_spherical(magmom_new_xyz)

        magmom_new[:, 0] = self.magmom_magnitude
        for i in range(self.structure.natom):
            if i not in self.mag_atoms:
                magmom_new[i, :] = 0.0

        properties = {'magmom': list(magmom_new.flatten()), 'energy': self.debug_evaluation(magmom_new)}

        if in_place:
            self.update_properties(entry_id, new_properties=properties)
            return entry_id
        else:
            return self.new_entry(magmom_new, active=False)

    def value(self, entry_id):
        """
        Return the energy value associated to the candidate with identifier 'entry_id'

        :param entry_id:
        :return:
        """
        entry = self.get_entry(entry_id, {'properties.energy': 1})
        if 'energy' in entry['properties']:
            return entry['properties']['energy']
        else:
            return None

    def str_entry(self, entry_id):
        entry = self.get_entry(entry_id, {'properties': 1})
        ret = '['
        for i in range(len(self.mag_atoms)):
            ret += ("[%8.5f %8.5f %8.5f] " % tuple(np.array(entry['properties']['magmom']).reshape((-1, 3))[i]))
        ret += '] '
        ret += 'Energy= %f' % entry['properties']['energy']
        return ret

    def get_duplicates(self, ids):
        return None

    def add_random(self):
        """
        Creates a new candidate with random orientiations for the magnetic moments of all the atoms in 'mag_atoms'

        :return:
        """
        n = self.structure.natom
        a = self.magmom_magnitude * np.ones(n)
        b = 2 * np.pi * np.random.random(n) - np.pi
        c = np.pi * np.random.random(n)
        magmom = np.vstack((a, b, c)).T
        for i in range(self.structure.natom):
            if i not in self.mag_atoms:
                magmom[i, :] = 0.0

        return self.new_entry(magmom), None

    def recover(self):
        data = self.get_population_info()
        if data is not None:
            self.mag_atoms = data['mag_atoms']
            self.distance_tolerance = data['distance_tolerance']
            self.name = data['name']
            self.magmom_magnitude = data['magmom_magnitude']
            self.source_dir = data['source_dir']
            self.incar_extra = data['incar_extra']

    def cross(self, ids):
        entry_id = ids[0]
        entry_jd = ids[1]
        entry = self.get_entry(entry_id, {'properties.magmom': 1})
        magmom_i = np.array(entry['properties']['magmom']).reshape((-1, 3))
        entry = self.get_entry(entry_jd, {'properties.magmom': 1})
        magmom_j = np.array(entry['properties']['magmom']).reshape((-1, 3))
        magmom_inew = np.zeros((self.structure.natom, 3))
        magmom_jnew = np.zeros((self.structure.natom, 3))
        for i in range(self.structure.natom):
            rnd = np.random.random()
            if rnd < 0.5:
                magmom_inew[i] = magmom_j[i]
                magmom_jnew[i] = magmom_i[i]
            else:
                magmom_inew[i] = magmom_i[i]
                magmom_jnew[i] = magmom_j[i]

        entry_id = self.new_entry(magmom_inew, active=True)
        entry_jd = self.new_entry(magmom_jnew, active=True)

        return entry_id, entry_jd

    def prepare_folder(self, entry_id, workdir, binary='vasp', source_dir='.'):

        if not os.path.isdir(workdir):
            os.mkdir(workdir)

        for i in ['KPOINTS', 'POSCAR', 'POTCAR']:
            if os.path.exists(workdir+os.sep+i):
                os.remove(workdir+os.sep+i)
            os.symlink(os.path.abspath(self.source_dir+os.sep+i), workdir+os.sep+i)

        input = read_incar(self.source_dir + os.sep + 'INCAR')
        magmom_sph = self.get_entry(entry_id, {'properties.magmom': 1})['properties']['magmom']
        magmom_car = spherical_to_cartesian(magmom_sph)
        input['MAGMOM'] = [float(x) for x in magmom_car.flatten()]
        input['M_CONSTR'] = [float(x) for x in magmom_car.flatten()]
        for i in self.incar_extra:
            input[i] = self.incar_extra[i]
        input.write(workdir + os.sep + 'INCAR')

    def collect_data(self, entry_id, workdir):
        if os.path.isfile(workdir + '/OUTCAR'):
            vo = VaspOutput(workdir + '/OUTCAR')
            if 'energy' in vo.final_data:
                if 'free_energy' in vo.final_data['energy']:
                    energy = vo.final_data['energy']['free_energy']
                    print('Uploading energy data for %s' % entry_id)
                    self.set_in_properties(entry_id, 'energy', energy)
                    return True
                else:
                    return False
            else:
                return False
        else:
            return False



import re
import subprocess


class FindSym:
    def __init__(self, structure, tolerance=0.1, executable='findsym'):

        self.structure = structure
        self.tolerance = tolerance
        self.executable = executable
        self.data = None

    def create_input(self, filename='findsym.inp'):

        wf = open(filename, 'w')

        wf.write('Input for FINDSYM, from filename\n')
        wf.write('%0.3E Tolerance\n' % self.tolerance)
        wf.write('1    form of lattice parameters: to be entered as vectors\n')
        counter = 1
        for i in self.structure.cell:
            wf.write('%12.8f %12.8f %12.8f Lattice vector %d\n' % (i[0], i[1], i[2], counter))
            counter += 1
        wf.write('P    unknown centering\n')
        wf.write('%d    number of atoms\n' % self.structure.natom)
        for i in self.structure.symbols:
            wf.write('%d ' % i)
        wf.write('Atom kinds\n')
        counter = 1
        for i in self.structure.reduced:
            wf.write('%24.16f %24.16f %24.16f Reduced Coordinates %d' % (i[0], i[1], i[2], counter))
            counter += 1

    def run(self):
        self.create_input()
        rf = open('findsym.inp')
        self.data = subprocess.check_output([self.executable], stdin=rf)
        rf.close()

    def write_cif(self, filename='structure.cif'):

        if self.data is None:
            self.run()

        findre = re.findall('# CIF file[\s\w.#,-]*', self.data)

        if len(findre) != 1:
            print('ERROR: Not CIF data was found on FindSym output')
            return
        else:
            cifdata = findre[0]

        wf = open(filename, 'w')
        wf.write(cifdata)
        wf.close()



import math
import sys

import numpy as np
from scipy.io import netcdf_file as _netcdf_file

if 'matplotlib' not in sys.modules:
    import matplotlib

    matplotlib.use('agg')
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from pychemia.code.abinit import InputVariables, AbiFiles
from pychemia.utils.periodic import covalent_radius
from pychemia.utils.constants import bohr_angstrom, angstrom_bohr


# 2D Plots for ABIPYTHON (Requires MATPLOTLIB)
def plot_history_energy(etotal, ekin, fcart, labels, bonds, filep):
    """
    Create a window with 2 plots, the total energy vs iterations
    and the magnitute of forces vs iterations
    """

    pp = PdfPages(filep)
    plt.ioff()

    # Page No 1
    plt.figure(figsize=(16, 10), dpi=100)

    left = 0.1
    width = 0.8
    bottom = 0.1

    if ekin is None:
        height = 0.40
    else:
        height = 0.3

    ax1 = plt.axes([left, bottom, width, height])
    ax1.plot(etotal, lw=2)
    ax1.grid(True)
    plt.ylabel("Total Energy")
    plt.xlabel("Iterations")

    if ekin is not None:
        bottom += height
        ax2 = plt.axes([left, bottom, width, height], sharex=ax1)
        ax2.semilogy(ekin, lw=2)
        ax2.grid(True)
        plt.ylabel("Ionic Kinetic Energy")
        # xticklabels = ax2.get_xticklabels()

    bottom += height
    ax3 = plt.axes([left, bottom, width, height], sharex=ax1)
    forces = fcart.transpose((1, 0, 2))
    # xticklabels = xticklabels + ax3.get_xticklabels()

    for i in range(len(forces)):
        list2 = []
        for j in range(len(forces[0])):
            list2.append(math.sqrt(sum(forces[i, j, :] * forces[i, j, :])))
        ax3.semilogy(list2, label=labels[i], lw=2)

    ax3.grid(True)
    ax3.axhline(0, color='black', lw=2)
    plt.ylabel("Forces")

    pp.savefig()

    # Page No 2
    plt.figure(figsize=(10, 16), dpi=100)

    left = 0.1
    width = 0.8
    height = 0.9 / float(len(forces))
    bottom = 0.95 - height
    plt.suptitle("Forces")

    for i in range(len(forces)):
        mag_force = []
        for j in range(len(forces[0])):
            mag_force.append(math.sqrt(sum(forces[i, j, :] * forces[i, j, :])))
        ax1 = plt.axes([left, bottom, width, height])
        ax1.semilogy(mag_force, label=labels[i], lw=2)
        ax1.grid(True)
        bottom -= height
        plt.ylabel(labels[i])
        (xmin, xmax) = ax1.get_xlim()
        if i < len(forces) - 1:
            ax1.set_xticklabels([])
        if xmax < 20:
            plt.xticks(range(int(xmax)))

    pp.savefig()

    # Page No 3
    plt.figure(figsize=(10, 16), dpi=100)

    bonds_dict = {}
    # t is time
    for t in range(len(bonds)):
        # print "Plotting bonds for time:",t
        # print "Number of bonds",len(bonds[t])
        for b in range(len(bonds[t])):

            bond = bonds[t][b]
            key = labels[bond[0]] + ' - ' + labels[bond[1]]
            if key in bonds_dict:
                bonds_dict[key].append([t, bond[2]])
            else:
                bonds_dict[key] = [[t, bond[2]]]

    left = 0.1
    width = 0.8
    nbonds = len(list(bonds_dict.keys()))
    height = 0.9 / float(nbonds)
    bottom = 0.95 - height
    plt.suptitle("Bonds")

    for i in bonds_dict:
        # print i

        ax1 = plt.axes([left, bottom, width, height])
        array = np.array(bonds_dict[i])
        ax1.plot(array[:, 0], bohr_angstrom * (array[:, 1] - array[0, 1]),
                 label='Initial=' + str(bohr_angstrom * array[0, 1]))
        ax1.grid(True)
        plt.ylabel(i)
        (xmin, xmax) = ax1.get_xlim()
        if i != list(bonds_dict.keys())[-1]:
            ax1.set_xticklabels([])
        if xmax < 20:
            plt.xticks(range(int(xmax)))
        bottom -= height
        plt.legend(loc=1)

    pp.savefig()

    pp.close()


def compute_bonds(typat, xcart, znucl):
    """
    Compute the bond lengths of all the atoms
    inside the unitary box (NEEDS EXTENSION TO
    OUTSIDE THE BOX)

    :param typat: (int, list) Type of atoms
    :param xcart: (numpy.ndarray) Cartesian positions
    :param znucl: (int, list) Atomic number for atoms in typat
    :return:
    """
    if isinstance(typat, int):
        lsttypat = [typat]
    else:
        lsttypat = typat
    if isinstance(znucl, (int, float)):
        lstznucl = [znucl]
    else:
        lstznucl = znucl

    covrad = [angstrom_bohr * covalent_radius(lstznucl[i - 1]) for i in lsttypat]
    bonds = []
    for t in range(len(xcart)):
        bonds1 = []
        for i in range(len(xcart[t])):
            for j in range(i + 1, len(xcart[t])):
                # print xcart[t][j]
                # print xcart[t][i]
                # Compute bond length between atoms i and j
                bl = math.sqrt(sum((xcart[t][j] - xcart[t][i]) ** 2))
                # print bl
                # print i,j
                # print covrad[i],covrad[j]
                if 1.35 * (covrad[i] + covrad[j]) > bl:
                    bonds1.append([i, j, bl, (xcart[t][j] - xcart[t][i]) / bl])
        bonds.append(bonds1)
    return bonds


def compute_angles(bonds, natom):
    print('natom', natom)
    itime = 0

    # Compute the number of bonds per atom
    nbonds = np.zeros(natom)
    for i in bonds[itime]:
        nbonds[i[0]] += 1
        nbonds[i[1]] += 1

    # Compute the angle for atoms with more than 1 bond
    for iatom in range(natom):
        if nbonds[iatom] > 1:
            index = []
            j = 0
            for b in bonds[itime]:
                if b[0] == iatom or b[1] == iatom:
                    index.append(j)
                j += 1
            print(index)

            for i in range(len(index)):
                for j in range(i + 1, len(index)):
                    print('i=', i)
                    print('j=', j)
                    # print nbonds


def get_history(abinitfile, dataset=""):
    if dataset == "":
        filep = abinitfile.basedir + "/" + abinitfile.files['tmpout'] + "_HIST"
    else:
        filep = abinitfile.basedir + "/" + abinitfile.files['tmpout'] + "_DS" + str(dataset) + "_HIST"
        # print filename
    # inp = InputVariables(abinitfile.get_input_filename())

    ret = _netcdf_file(filep, 'r', mmap=False)

    return ret


def plot_history(abinitfile, dataset=""):
    history = get_history(abinitfile, dataset)
    av = InputVariables(abinitfile.get_input_filename())

    if dataset == "":
        filep = abinitfile.basedir + "/" + abinitfile.files['tmpout'] + ".pdf"
    else:
        filep = abinitfile.basedir + "/" + abinitfile.files['tmpout'] + "_DS" + str(dataset) + ".pdf"

    xcart = history.variables['xcart'][:]
    fcart = history.variables['fcart'][:]
    # rprimd = history.variables['rprimd'][:]
    etotal = history.variables['etotal'][:]
    if 'ekin' in history.variables:
        ekin = history.variables['ekin'][:]
        if max(history.variables['ekin'][:]) == 0.0:
            ekin = None
    else:
        ekin = None

    # Getting Labels of atoms
    labels = [av.atom_name(i) for i in range(av.get_value('natom', dataset))]
    znucl = av.get_value('znucl', idtset=dataset)
    typat = av.get_value('typat', idtset=dataset)

    bonds = compute_bonds(typat, xcart, znucl)
    # natom = len(xcart[0])
    # ComputeAngles(bonds,natom)

    plot_history_energy(etotal, ekin, fcart, labels, bonds, filep)


if __name__ == '__main__':

    # Script starts from here
    if len(sys.argv) < 2:
        print('No option specified.')
        sys.exit()

    filename = sys.argv[1]
    if len(sys.argv) == 3:
        idtset = int(sys.argv[2])
    else:
        idtset = ""

    abifile = AbiFiles(filename)
    plot_history(abifile, idtset)



#!/usr/bin/env python

import os
import scipy.io
from mayavi import mlab
import numpy as np
import pychemia
import pychemia.io.xyz


class OctopusVisualization:
    def __init__(self, fig, fs, gs, bs):
        """
        Creates a container for the
        sources of a visualization
        """
        self.fig = fig  # Figure
        self.fs = fs  # Field sources
        self.gs = gs  # Atomic Geometry sources
        self.bs = bs  # Cell box sources


def _compute_box(b):
    box = np.array([[0, 0, 0], [b[0], 0, 0], [b[0], b[1], 0],
                    [0, b[1], 0], [0, 0, 0], [0, 0, b[2]],
                    [0, b[1], b[2]], [b[0], b[1], b[2]],
                    [b[0], 0, b[2]], [0, 0, b[2]]])

    x0 = box[:, 0]
    y0 = box[:, 1]
    z0 = box[:, 2]

    x1 = np.array([b[0], b[0]])
    y1 = np.array([b[1], b[1]])
    z1 = np.array([0, b[2]])

    return x0, y0, z0, x1, y1, z1


def _read_netcdf(filename):
    if not os.path.isfile(filename):
        print('ERROR: Non existing file: ' + filename)
        exit(1)
    data = scipy.io.netcdf_file(filename)
    rdata = data.variables['rdata'][:]
    pos = data.variables['pos'][:]
    data.close()
    return np.abs(rdata), pos


def oct_visualize(path='static',
                  spin=None,
                  spin_pol=None,
                  size=(640, 480),
                  bgcolor=(0, 0, 0),
                  structure=True,
                  box=True,
                  figname='figure.png',
                  view=None,
                  visual=None):
    if spin is None:
        spin = [True, True]

    if visual is None:
        fig = mlab.figure(size=size, bgcolor=bgcolor)
    else:
        fig = visual.fig

    if spin_pol is None:
        inp = pychemia.code.octopus.InputVariables('inp')
        if inp.variables['SpinComponents'] == 'spin_polarized':
            spin_pol = True
        else:
            spin_pol = False

    rdata = []
    pos = []
    if not spin_pol:
        r, p = _read_netcdf(path + '/density.ncdf')
        rdata.append(r)
        pos.append(p)
        b = list(np.array(rdata).shape)
    else:
        if spin[0]:
            r, p = _read_netcdf(path + '/density-sp1.ncdf')
            rdata.append(r)
            pos.append(p)
            b = list(r.shape)
            pos = p
        if spin[1]:
            r, p = _read_netcdf(path + '/density-sp2.ncdf')
            rdata.append(r)
            pos.append(p)
            b = list(r.shape)
            pos = p

        if spin[0] is True and spin[1] is True and not np.all(pos[0] == pos[1]):
            print('ERROR box not consistent on', path)
            exit()

    if visual is None:
        fs = []
        if not spin_pol:
            fs1 = mlab.pipeline.scalar_field(np.log(rdata))
            mlab.pipeline.volume(fs1, vmin=-4, vmax=-1)
            fs.append(fs1)
        else:
            if spin[0]:
                fs1 = mlab.pipeline.scalar_field(np.log(rdata1))
                mlab.pipeline.volume(fs1, vmin=-4, vmax=-1, color=(1, 0, 0))
                fs.append(fs1)
            if spin[1]:
                fs2 = mlab.pipeline.scalar_field(np.log(rdata2))
                mlab.pipeline.volume(fs2, vmin=-4, vmax=-1, color=(0, 0, 1))
                fs.append(fs2)
    else:
        count = 0
        if not spin_pol:
            visual.fs[0].mlab_source.scalars = np.log(rdata)
        else:
            if spin[0]:
                visual.fs[count].mlab_source.scalars = np.log(rdata1)
                count += 1
            if spin[1]:
                visual.fs[count].mlab_source.scalars = np.log(rdata2)

    if structure:
        geometry_file = pychemia.io.xyz.load(path + '/geo.xyz')

        orig = pos[:, 0]
        delta = pos[:, 1]

        x = (geometry_file.positions[:, 0] - orig[0]) / delta[0]
        y = (geometry_file.positions[:, 1] - orig[1]) / delta[1]
        z = (geometry_file.positions[:, 2] - orig[2]) / delta[2]
        cr = pychemia.utils.periodic.covalent_radius(geometry_file.symbols)

        if visual is None:
            gs = mlab.points3d(x, y, z, cr, scale_factor=5)
        else:
            visual.gs.mlab_source.set(x=x, y=y, z=z)
    else:
        gs = None

    if visual is None:
        bs = []
        if box:
            x0, y0, z0, x1, y1, z1 = _compute_box(b)
            ze = np.zeros(2)

            bb1 = mlab.plot3d(x0, y0, z0, tube_radius=.15, color=(1, 1, 1))
            bb2 = mlab.plot3d(ze, y1, z1, tube_radius=.15, color=(1, 1, 1))
            bb3 = mlab.plot3d(x1, y1, z1, tube_radius=.15, color=(1, 1, 1))
            bb4 = mlab.plot3d(x1, ze, z1, tube_radius=.15, color=(1, 1, 1))

            bs = [bb1, bb2, bb3, bb4]
    else:
        if box:
            x0, y0, z0, x1, y1, z1 = _compute_box(b)
            ze = np.zeros(2)
            visual.bs[0].start()
            visual.bs[1].start()
            visual.bs[2].start()
            visual.bs[3].start()

            visual.bs[0].mlab_source.set(x=x0, y=y0, z=z0, tube_radius=.15, color=(1, 1, 1))
            visual.bs[1].mlab_source.set(x=ze, y=y1, z=z1, tube_radius=.15, color=(1, 1, 1))
            visual.bs[2].mlab_source.set(x=x1, y=y1, z=z1, tube_radius=.15, color=(1, 1, 1))
            visual.bs[3].mlab_source.set(x=x1, y=ze, z=z1, tube_radius=.15, color=(1, 1, 1))

        else:
            try:
                visual.bs[0].stop()
                visual.bs[1].stop()
                visual.bs[2].stop()
                visual.bs[3].stop()
            except ValueError:
                pass

    if visual is None:
        visual = OctopusVisualization(fig, fs, gs, bs)

    if view is None:
        mlab.view(45, 55, 500, np.array([b[0] / 2, b[1] / 2, b[2] / 2]))
    else:
        mlab.view(view[0], view[1], view[2], view[3])

    mlab.savefig(figname)

    return visual



from __future__ import print_function
import os
import socket
import time
from multiprocessing import Process
from pychemia import pcm_log, HAS_PYMONGO

if HAS_PYMONGO:
    from pychemia.db import get_database


class DirectEvaluator:

    def __init__(self, db_settings, dbnames, source_dir, is_evaluated, worker, worker_args=None, nconcurrent=1,
                 evaluate_failed=False, evaluate_all=False, sleeping_time=120):
        """
        DirectEvaluator is a class to manage the execution of a function 'worker' for entries on a list of PyChemiaDB
         databases.
        The execution of each worker occurs directly on the machine executing the code, and the class controls the
        number of concurrent execution using the variable 'nconcurrent'

        :param db_settings: Common database settings for all the databases that will be processed by this evaluator.
                            All databases should share common settings like server name, user and password, ssl and
                            replicaset settings.
        :param dbnames: List of database names, the common settings are stored in db_settings
        :param source_dir: Path to directory where the candidates will be evaluated.
        :param is_evaluated: Function to decide if one entry is evaluable
        :param worker: Function applied to each candidate on the databases that are not considered evaluated.
        :param worker_args: Arguments for the worker function. (Usually a dictionary that depends on the arguments
                            needed by worker)
        :param nconcurrent: Number of concurrent executions allowed. Their number usually should be the number of cores
                            available on the machine. (default: 1)
        :param evaluate_failed: Boolean to decide if candidates marked as failed should be evaluated again.
        :param evaluate_all: Boolean to decide is all candidates are evaluated regardless of the outcome of the function
                is_evaluated. (default: False)
        :param sleeping_time: Time in seconds before try to search for new candidates to evaluate (default: 120 seconds)
        """
        self.db_settings = db_settings
        self.dbnames = dbnames
        self.source_dir = source_dir
        self.is_evaluated = is_evaluated
        self.worker = worker
        self.worker_args = worker_args
        self.nconcurrent = nconcurrent
        self.evaluate_failed = evaluate_failed
        self.evaluate_all = evaluate_all
        self.sleeping_time = sleeping_time

    def unlock_all(self):
        """
        Checking all databases and unlocking all entries.

        :return: None
        """
        for idb in self.dbnames:
            db_settings = dict(self.db_settings)
            db_settings['name'] = idb
            pcdb = get_database(db_settings)
            print('Database contains: %d entries' % pcdb.entries.count())

            for entry in pcdb.entries.find({}):
                pcdb.unlock(entry['_id'], name=socket.gethostname())
            print('Number of entries: ', pcdb.entries.count())

    def get_list_candidates(self):
        """
        Scan all databases looking for candidates for evaluation

        :return: A list of pairs, each pair contains the name of the database
                    and candidate identifier.
        """
        ret = []
        for idb in self.dbnames:
            print(idb)
            db_settings = dict(self.db_settings)
            db_settings['name'] = idb
            pcdb = get_database(db_settings)

            for entry in pcdb.entries.find({}, {'_id': 1}):
                entry_id = entry['_id']
                if not self.is_evaluated(pcdb, entry_id, self.worker_args) or self.evaluate_all:
                    pcm_log.debug('Adding entry %s from db %s' % (str(entry_id), pcdb.name))
                    ret.append([idb, entry_id])
        print('Found %d entries to evaluate' % len(ret))
        return ret

    def run(self):
        """
        Continuously search for suitable candidates to evaluation among a list of databases.

        :return:
        """

        procs = []
        ids_running = []

        # Creating a list to store the 'nconcurrent' running jobs
        for i in range(self.nconcurrent):
            procs.append(None)
            ids_running.append(None)

        self.unlock_all()

        # Main loop looking permanently for candidates for evaluation
        while True:

            to_evaluate = self.get_list_candidates()

            index = 0
            currently_evaluating = 0
            for j in range(self.nconcurrent):
                if procs[j] is not None and procs[j].is_alive():
                    currently_evaluating += 1
            print('Candidates to evaluate: %d  Candidates in evaluation: %d' % (len(to_evaluate), currently_evaluating))

            while index < len(to_evaluate):

                db_settings = dict(self.db_settings)
                # The first component of each pair in to_evaluate is the name of the database
                dbname = to_evaluate[index][0]
                db_settings['name'] = dbname
                pcdb = get_database(db_settings)
                # The second component of each pair in to_evaluate is the entry_id
                entry_id = to_evaluate[index][1]

                for j in range(self.nconcurrent):
                    if procs[j] is None or not procs[j].is_alive():
                        ids_running[j] = None

                if entry_id in ids_running:
                    print('Already executing: %s' % entry_id)
                    index += 1
                    continue
                else:
                    print('DB: %10s Entry: %s' % (dbname, entry_id))

                if not os.path.exists(self.source_dir + os.sep + dbname):
                    os.mkdir(self.source_dir + os.sep + dbname)

                slot = None
                while True:
                    for j in range(self.nconcurrent):
                        if procs[j] is None or not procs[j].is_alive():
                            slot = j
                            break
                    if slot is None:
                        time.sleep(self.sleeping_time)
                    else:
                        break

                # The function is_evaluated needs two arguments, the database object and entry identifier and
                # must return a boolean to decide if the candidate should be evaluated.
                if not self.is_evaluated(pcdb, entry_id, self.worker_args) or self.evaluate_all:
                    pcm_log.debug('Evaluable: %s:%s. Relaxing entry %d of %d Slot: %d' % (dbname,
                                                                                          str(entry_id),
                                                                                          index,
                                                                                          len(to_evaluate), slot))
                    ids_running[slot] = entry_id
                    workdir = self.source_dir + os.sep + dbname + os.sep + str(entry_id)
                    if not os.path.exists(workdir):
                        os.mkdir(workdir)
                    pcm_log.debug('Launching for %s id: %s' % (pcdb.name, str(entry_id)))

                    # This is the actual call to the worker, it must be a function with 4 arguments:
                    # The database settings, the entry identifier, the working directory and arguments for the worker
                    procs[slot] = Process(target=self.worker, args=(db_settings, entry_id, workdir, self.worker_args))
                    procs[slot].start()
                    time.sleep(1)
                else:
                    pcm_log.debug('Not evaluable: %s' % str(entry_id))
                index += 1
            time.sleep(self.sleeping_time)



import os
from .incar import InputVariables
from .poscar import write_poscar


def write_from_queue(queue, entry_id, destination=None):
    if destination is None:
        dest = '.'
    elif os.path.isfile(destination):
        dest = os.path.dirname(os.path.abspath(destination))
    elif not os.path.exists(destination):
        os.mkdir(destination)
        dest = destination
    elif os.path.isdir(destination):
        dest = destination
    else:
        raise ValueError('Destination not valid')

    st = queue.get_input_structure(entry_id)
    inpvars = queue.get_input_variables(entry_id)

    vi = InputVariables()
    for i in inpvars:
        vi[i] = inpvars[i]

    write_poscar(st, dest + os.sep + 'POSCAR')
    vi.write(dest + os.sep + 'INCAR')

    queue.write_input_files(entry_id, dest)



import os
import numpy as np


class VaspDoscar:
    def __init__(self, filename='DOSCAR'):

        if not os.path.isfile(filename):
            raise ValueError('ERROR: DOSCAR file not found')

        self.filename = filename

        self.doscar = self.parse_doscar(self.filename)

        if 'projected' in self.doscar:
            self.has_projected = True
            self.nions = len(self.doscar['projected'])
        else:
            self.has_projected = False
            self.nions = None

        self.ndos, self.total_ncols = self.doscar['total'].shape

        if self.total_ncols == 5:
            self.is_spin_polarized = True
        else:
            self.is_spin_polarized = False

        self.total_dos = self._dos_dict()

        if self.has_projected:
            self.projected_dos = self._proj_dos_dict()

    def _dos_dict(self):

        ret = {'energy': self.doscar['total'][:, 0]}

        if self.is_spin_polarized:
            ret['dos_up'] = self.doscar['total'][:, 1]
            ret['dos_down'] = self.doscar['total'][:, 2]
            ret['integrated_dos_up'] = self.doscar['total'][:, 3]
            ret['integrated_dos_down'] = self.doscar['total'][:, 4]
        else:
            ret['dos'] = self.doscar['total'][:, 1]
            ret['integrated_dos'] = self.doscar['total'][:, 2]
        return ret

    def _proj_dos_dict(self):

        ret = {}
        for i in range(self.nions):
            ret['proj'] = {'energy': self.doscar['projected'][i][:, 0]}
            if self.is_spin_polarized:
                ret['ncols'] = self.doscar['projected'][i].shape[1]
        return ret

    @staticmethod
    def parse_doscar(filename):

        if not os.path.isfile(filename):
            raise ValueError('ERROR: DOSCAR file not found')

        rf = open(filename)
        data = rf.readlines()
        rf.close()

        if len(data) < 5:
            raise ValueError('DOSCAR seems truncated')

        # Skipping the first lines of header
        iline = 5

        header = [float(x) for x in data[iline].strip().split()]
        ndos = int(header[2])
        iline += 1

        total_dos = [[float(x) for x in y.split()] for y in data[iline:iline + ndos]]
        total_dos = np.array(total_dos)

        iline += ndos

        # In case there are more lines of data, they are the projected DOS
        if len(data) > iline:

            projected_dos = []

            while iline < len(data):
                header = [float(x) for x in data[iline].strip().split()]
                ndos = int(header[2])
                iline += 1
                tmp_dos = [[float(x) for x in y.split()] for y in data[iline:iline + ndos]]
                projected_dos.append(tmp_dos)
                iline += ndos

            projected_dos = np.array(projected_dos)

            return {'total': total_dos, 'projected': projected_dos}

        else:

            return {'total': total_dos}



import pychemia

try:
    import spglib as spg
    USE_SPGLIB = True
except ImportError:
    USE_SPGLIB = False
    # from pyspglib import spglib as spg

"""
Interface with the Atomic Simulation Environment (ASE)
Convert ase Atoms objects into pymatdis crystal objects
"""

__author__ = "Guillermo Avendano-Franco"
__copyright__ = "Copyright 2014"
__version__ = "0.1"
__maintainer__ = "Guillermo Avendano-Franco"
__email__ = "gtux.gaf@gmail.com"
__status__ = "Development"
__date__ = "March 31, 2014"

try:
    import ase.atoms
    import ase.io
except ImportError:
    print('The module ase could not be imported')
    raise ImportError


def cif2structure(filename, primitive=False, symprec=0.001):
    """
    Read a given file  as a cif file, convert it into a
    ase atoms object and finally into a pychemia
    structure

    :param filename: (str) Filename of the CIF file that will be read
    :param primitive: (boolean) if primitive cell should be returned
    :param symprec: (float) Desired precision for computing the primitive cell
    :return:
    """
    aseatoms = ase.io.read(filename)
    if primitive and USE_SPGLIB:
        lattice, scaled_positions, numbers = spg.find_primitive(aseatoms, symprec)
        if lattice is not None and scaled_positions is not None and numbers is not None:
            fin_atoms = ase.atoms.Atoms(cell=lattice, scaled_positions=scaled_positions, numbers=numbers, pbc=True)
        else:
            fin_atoms = aseatoms
    else:
        fin_atoms = aseatoms
    ret = ase2pychemia(fin_atoms)
    return ret


def ase2pychemia(aseatoms):
    """
    Converts an ase atoms object into a pychemia
    structure object
    """
    cell = aseatoms.get_cell()
    an = aseatoms.get_atomic_numbers()
    symbols = pychemia.utils.periodic.atomic_symbol(an)
    positions = aseatoms.get_positions()
    return pychemia.Structure(cell=cell, positions=positions, symbols=symbols)


def pychemia2ase(structure):
    """
    Converts an pychemia structure into a ase atoms object

    :param structure: (pychemia.Structure) PyChemia Structure to convert into a ASE Atoms object
    :return:
    """
    cell = structure.cell
    scaled_positions = structure.reduced
    symbols = structure.symbols
    return ase.atoms.Atoms(cell=cell, scaled_positions=scaled_positions, symbols=symbols)



"""
PyChemia works with atomic structures, this module provides two main classes to store structure information.
The class 'Structure' and the class 'Composition' are widely used for the entire package.

"""

from .composition import Composition
from .structure import Structure


# __all__ = filter(lambda s: not s.startswith('_'), dir())



#!/usr/bin/env python
import sys
import logging
import itertools
from multiprocessing import Pool
import pychemia
from pychemia.utils.periodic import atomic_symbol

try:
    from qmpy import Entry
except ImportError:
    Entry = None
    print("Could not import 'qmpy' as needed to interface with the OQMD database")
    exit(1)

version = 0.1
jump = 10000


def help_info():
    print(""" Create or Update a PyChemia Database from the OQMD Database (www.oqmd.org)

   Use:

       OQMD2PyChemia.py --dbname 'MongoDB Database name'
                                [--host localhost ] [--port 27017]
                                [--user None ] [--passwd None] [--ssl]

   """)


def run_one(a):
    """
    Take one OQMD 'Entry' object, search all the calculations associated and take the best calculation
    in order to insert its data into the PyChemia Database

    :param a: OQMD Entry object
    :return:
    """
    print('Entry: %6d  Number of Calculations: %3d  Energies: %s' % (a.id, a.calculation_set.count(),
                                                                     str([c.energy for c in a.calculation_set.all()])))
    energy = 1E10
    best_calculation = None
    for c in a.calculation_set.all():
        if c.energy is not None and c.energy < energy:
            best_calculation = c
            energy = c.energy

    if best_calculation is None:
        return None, None

    cell = best_calculation.output.cell.T
    symbols = atomic_symbol(best_calculation.output.atomic_numbers)
    reduced = best_calculation.output.coords

    structure = pychemia.Structure(cell=cell, symbols=symbols, reduced=reduced)
    structure_id = best_calculation.output_id
    entry_id = a.id
    calculation_id = best_calculation.id
    energy_pa = best_calculation.energy_pa
    energy = best_calculation.energy
    band_gap = best_calculation.band_gap
    settings = best_calculation.settings
    try:
        spacegroup_number = best_calculation.output.spacegroup.number
    except ValueError:
        spacegroup_number = None

    symm = pychemia.crystal.CrystalSymmetry(structure)
    sym2 = symm.number(1E-2)

    properties = {'oqmd': {'structure_id': structure_id,
                           'entry_id': entry_id,
                           'calculation_id': calculation_id,
                           'energy_pa': energy_pa,
                           'energy': energy,
                           'band_gap': band_gap,
                           'settings': settings,
                           'spacegroup_number': spacegroup_number},
                  'spacegroup_number': {'value': sym2, 'symprec': 1E-2}}

    return structure, properties


def getter(entry_ids, db_settings, current, start=0):
    pcdb = pychemia.db.get_database(db_settings)

    ret = []

    index = 0
    n = 0

    initial = start * jump
    final = min(start * jump + jump, len(entry_ids))
    print('Process: %2d Processing from %6d to %6d' % (start, initial, final))

    for a_id in entry_ids[initial:final]:

        if a_id not in current[index:]:
            ret.append(a_id)
            n += 1
        else:
            index = current.index(a_id)
            # Removing duplicated entries
            if index < len(current) and current[index + 1] == a_id:
                print('We found at least one duplicate!')
                duplicate = False
                for entry in pcdb.db.pychemia_entries.find({'properties.oqmd.entry_id': a_id}):
                    if duplicate:
                        print('Removing PyChemiaDB entry: %s' % str(entry['_id']))
                        pcdb.db.pychemia_entries.remove({'_id': entry['_id']})
                    duplicate = True

    print('Process: %2d Entries missing: %3d' % (start, len(ret)))

    return ret


def setter(pcdb, to_insert):
    for entry_id in to_insert:
        a = Entry.objects.get(id=entry_id)
        structure, properties = run_one(a)
        if structure is not None:
            pcdb.insert(structure, properties=properties)


def getter_star(a_b):
    return getter(*a_b)


if __name__ == '__main__':

    logging.basicConfig(level=logging.DEBUG)
    logger = logging.getLogger('pychemia')
    logger.addHandler(logging.NullHandler())
    logger.setLevel(logging.DEBUG)

    # Script starts from here
    if len(sys.argv) < 2:
        help_info()
        sys.exit(1)

    dbname = None
    host = 'localhost'
    port = 27017
    user = None
    passwd = None
    path = None
    ssl = False

    for i in range(1, len(sys.argv)):
        if sys.argv[i].startswith('--'):
            option = sys.argv[i][2:]
            # fetch sys.argv[1] but without the first two characters
            if option == 'version':
                print(version)
                sys.exit()
            elif option == 'help':
                help_info()
                sys.exit()
            elif option == 'dbname':
                dbname = sys.argv[i + 1]
            elif option == 'host':
                host = sys.argv[i + 1]
            elif option == 'port':
                dbname = int(sys.argv[i + 1])
            elif option == 'user':
                user = sys.argv[i + 1]
            elif option == 'passwd':
                passwd = sys.argv[i + 1]
            elif option == 'ssl':
                ssl = True
            else:
                print('Unknown option. --' + option)

    if dbname is None:
        help_info()
        sys.exit(1)

    db_settings = {'name': dbname, 'host': host, 'port': port, 'ssl': ssl}
    if user is not None:
        if passwd is None:
            raise ValueError('Password is mandatory if user is entered')
        db_settings['user'] = user
        db_settings['passwd'] = passwd

    pcdb = pychemia.db.get_database(db_settings)

    nitems = pcdb.entries.count()
    print('Number of entries in the current PyChemia Database: %d' % nitems)

    current = []
    for entry in pcdb.db.pychemia_entries.find({'properties.oqmd.entry_id': {'$exists': True}}):
        current.append(entry['properties']['oqmd']['entry_id'])
    current.sort()
    print('Number of entries coming from OQMD: %d' % len(current))

    queryset = Entry.objects.all()
    entry_ids = [entry.id for entry in queryset]
    print('Number of entries in OQMD: %d' % len(entry_ids))

    pool = Pool(processes=6)

    argus = []
    a_args = range(len(entry_ids) / jump)

    ret = pool.map(getter_star, itertools.izip(itertools.repeat(entry_ids),
                                               itertools.repeat(db_settings),
                                               itertools.repeat(current), a_args))

    to_insert = []
    for i in ret:
        for j in i:
            to_insert.append(j)

    pool.close()

    setter(pcdb, to_insert)



from __future__ import unicode_literals
import spglib as spg
from pychemia.utils.serializer import generic_serializer
from pychemia.utils.computing import deep_unicode


def spglib_version():
    from pychemia import Structure
    from . import CrystalSymmetry

    # Testing version of spglib
    st = Structure(symbols=['H'])
    symm = CrystalSymmetry(st)
    ret = spg.spglib.spg.dataset(symm.transposed, symm.reduced, symm.numbers, 1e-5, -1.0)
    if type(ret[3]) is list:
        HAS_SPGLIB = False
        version = "%d.%d.%d" % spg.get_version()
        print('SPGLIB current version is %s, please install spglib > 1.9' % version)
    else:
        HAS_SPGLIB = True
    return HAS_SPGLIB


class CrystalSymmetry(object):
    """
    Takes a pychemia.Structure object and creates an object with methods
    to identify symmetry groups and other symmetry related operations.
    Uses spglib to perform various symmetry finding operations.
    """

    def __init__(self, structure):
        """
        Creates a new StructureSymmetry object for a given structure
        This class allows interaction with spglib for several operations related
        to symmetry

        :param structure:

        Example:
        >>> import pychemia
        >>> a = 4.05
        >>> b = a/2
        >>> fcc = pychemia.Structure(symbols=['Au'], cell=[[0, b, b], [b, 0, b], [b, b, 0]], periodicity=True)
        >>> symm = pychemia.crystal.CrystalSymmetry(fcc)
        >>> symm.number()
        225
        >>> symm.symbol() == u'Fm-3m'
        True

        """
        assert structure.is_crystal
        assert structure.is_perfect
        self.structure = structure
        # Spglib"s convention for the lattice definition is the transpose of cell
        # self._transposed_cell = structure.cell.transpose().copy()
        self.spglib_lattice = generic_serializer(structure.cell.copy())
        # Spglib requires numpy floats.
        # self._transposed_cell = np.array(self._transposed_cell, dtype='double', order='C')
        # self._reduced = np.array(structure.reduced, dtype='double', order='C')
        self.spglib_positions = generic_serializer(structure.reduced)
        # Get a list of indices for each atom in structure
        # Indices starting in 1
        # self._numbers = np.array([structure.species.index(x) + 1 for x in structure.symbols], dtype='intc')
        self.spglib_numbers = [structure.species.index(x) + 1 for x in structure.symbols]

    # @property
    # def transposed(self):
    #     return self._transposed_cell
    #
    # @property
    # def reduced(self):
    #     return self._reduced
    #
    # @property
    # def numbers(self):
    #     return self._numbers

    @property
    def spglib_cell(self):
        return self.spglib_lattice, self.spglib_positions, self.spglib_numbers

    def get_spacegroup(self, symprec=1e-5):
        return spg.get_spacegroup(cell=self.spglib_cell, symprec=symprec)

    def get_symmetry(self, symprec=1e-5):
        return spg.get_symmetry(cell=self.spglib_cell, symprec=symprec)

    def get_symmetry_dataset(self, symprec=1e-5):
        return spg.get_symmetry_dataset(cell=self.spglib_cell, symprec=symprec)

    def number(self, symprec=1e-5):
        return int(self.get_symmetry_dataset(symprec)['number'])

    def symbol(self, symprec=1e-5):
        return deep_unicode(self.get_symmetry_dataset(symprec)['international'])

    def hall_number(self, symprec=1e-5):
        return self.get_symmetry_dataset(symprec)['hall_number']

    def get_new_structure(self, spglib_cell):
        from pychemia import Structure
        cell = spglib_cell[0]
        reduced = spglib_cell[1]
        symbols = [self.structure.species[x - 1] for x in spglib_cell[2]]
        return Structure(cell=cell, reduced=reduced, symbols=symbols)

    def refine_cell(self, symprec=1e-5):
        """
        Refine a pychemia Structure using the tolerances and return a new structure in a Bravais lattice

        :param symprec: (float) Tolerance of distance between atomic positions and between lengths of lattice vectors
                        to be tolerated in the symmetry finding.
        :return: A new pychemia Structure in a Bravais lattice

        :rtype : (pychemia.Structure)

        Example:
        >>> import pychemia
        >>> a = 4.05
        >>> b = a/2
        >>> fcc = pychemia.Structure(symbols=['Au'],
        ...       cell=[[0, b+1E-5, b-1E-5], [b+1E-5, 0, b-1E-5], [b+1E-5, b-1E-5, 0]], periodicity=True)
        >>> symm = pychemia.crystal.CrystalSymmetry(fcc)
        >>> symm.number()
        2
        >>> symm.symbol() == u'P-1'
        True
        >>> fcc2 = symm.refine_cell(symprec=1E-3)
        >>> symm2 = pychemia.crystal.CrystalSymmetry(fcc2)
        >>> symm2.number()
        225
        >>> symm2.symbol() == u'Fm-3m'
        True

        """
        new_spglib_cell = spg.refine_cell(self.spglib_cell, symprec=symprec)
        return self.get_new_structure(new_spglib_cell)

    def find_primitive(self, symprec=1e-5):
        new_spglib_cell = spg.find_primitive(self.spglib_cell, symprec=symprec)
        return self.get_new_structure(new_spglib_cell)

    def standarize_cell(self, symprec=1e-5):
        new_spglib_cell = spg.standarize_cell(self.spglib_cell, symprec=symprec)
        return self.get_new_structure(new_spglib_cell)

    def get_space_group_type(self, symprec=1e-5):
        return spg.get_spacegroup_type(self.hall_number(symprec=symprec))

    def crystal_system(self, symprec=1e-5):

        num = self.number(symprec)

        if num < 3:
            return u'Triclinic'
        elif num < 16:
            return u'Monoclinic'
        elif num < 75:
            return u'Orthorhombic'
        elif num < 143:
            return u'Tetragonal'
        elif num < 168:
            return u'Trigonal'
        elif num < 195:
            return u'Hexagonal'
        else:
            return u'Cubic'

    def symmetrize(self, initial_symprec=0.01, final_symprec=0.1, delta_symprec=0.01):
        if self.structure.natom == 1:
            return self.structure.copy()
        prec = initial_symprec
        while prec < final_symprec:
            if self.number(symprec=prec) > self.number(symprec=initial_symprec):
                break
            else:
                prec += delta_symprec
        if prec > final_symprec:
            prec = final_symprec
        new_bravais = self.refine_cell(symprec=prec)
        sym2 = CrystalSymmetry(new_bravais)
        return sym2.find_primitive()

        # def get_symmetry_dataset(self, symprec=1e-5, angle_tolerance=-1.0):
        #
        #     keys = ('number',
        #             'hall_number',
        #             'international',
        #             'hall',
        #             'transformation_matrix',
        #             'origin_shift',
        #             'rotations',
        #             'translations',
        #             'wyckoffs',
        #             'equivalent_atoms',
        #             'brv_lattice',
        #             'brv_types',
        #             'brv_positions')
        #     dataset = {}
        #     for key, data in zip(keys, spg.spglib.spg.dataset(self._transposed_cell,
        #                                                       self._reduced,
        #                                                       self._numbers,
        #                                                       symprec,
        #                                                       angle_tolerance)):
        #         dataset[key] = data
        #
        #     dataset['international'] = dataset['international'].strip()
        #     dataset['hall'] = dataset['hall'].strip()
        #     dataset['transformation_matrix'] = np.array(
        #         dataset['transformation_matrix'], dtype='double', order='C')
        #     dataset['origin_shift'] = np.array(dataset['origin_shift'], dtype='double')
        #     dataset['rotations'] = np.array(dataset['rotations'],
        #                                     dtype='intc', order='C')
        #     dataset['translations'] = np.array(dataset['translations'],
        #                                        dtype='double', order='C')
        #     letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
        #     dataset['wyckoffs'] = [letters[x] for x in dataset['wyckoffs']]
        #     dataset['equivalent_atoms'] = np.array(dataset['equivalent_atoms'],
        #                                            dtype='intc')
        #     dataset['brv_lattice'] = np.array(np.transpose(dataset['brv_lattice']),
        #                                       dtype='double', order='C')
        #     dataset['brv_types'] = np.array(dataset['brv_types'], dtype='intc')
        #     dataset['brv_positions'] = np.array(dataset['brv_positions'],
        #                                         dtype='double', order='C')
        #
        #     return dataset
        #
        # def get_spacegroup(self, symprec=1e-5, angle_tolerance=-1.0):
        #     """
        #     Return space group in international table symbol and number
        #     as a string.
        #
        #     :param symprec: (float) Symmetry precision
        #     :param angle_tolerance: (float) angle tolerance for spglib internal routine
        #     :return:
        #     """
        #
        #     dataset = self.get_symmetry_dataset(symprec=symprec, angle_tolerance=angle_tolerance)
        #     return "%s (%d)" % (dataset['international'], dataset['number'])
        #
        # def spacegroup(self, symprec=1e-5, angle_tolerance=-1.0):
        #     """
        #     Computes the space group for the structure with a given
        #     precision in distances (symprec) and angle tolerance in degrees (angle_tolerance)
        #
        #     :param symprec: (float) Tolerance of distance between atomic positions and between lengths of
        #                             lattice vectors
        #     to be tolerated in the symmetry finding.
        #     :param angle_tolerance: (float) Tolerance of angle between lattice vectors in degrees to be
        #                                     tolerated in the symmetry finding.
        #
        #     :return: The space group with the symbol and number as a string
        #     :rtype : (str)
        #     """
        #
        #     return self.get_spacegroup(symprec, angle_tolerance)
        #
        # def symbol(self, symprec=1e-5, angle_tolerance=-1.0):
        #     """
        #     Computes the space group symbol for the structure with a given
        #     precision in distances (symprec) and angle tolerance in degrees (angle_tolerance)
        #
        #     :param symprec: (float) Tolerance of distance between atomic positions and between lengths of
        #                             lattice vectors to be tolerated in the symmetry finding.
        #     :param angle_tolerance: (float) Tolerance of angle between lattice vectors in degrees to be
        #                                     tolerated in the symmetry finding.
        #
        #     :return: The space group symbol as a string
        #     :rtype : (str)
        #     """
        #     return self.spacegroup(symprec, angle_tolerance).split()[0]
        #
        # def number(self, symprec=1e-5, angle_tolerance=-1.0):
        #     """
        #     Computes the space group number for the structure with a given
        #     precision in distances (symprec) and angle tolerance in degrees (angle_tolerance)
        #
        #     :param symprec: (float) Tolerance of distance between atomic positions and between lengths
        #                             of lattice vectors to be tolerated in the symmetry finding.
        #     :param angle_tolerance: (float) Tolerance of angle between lattice vectors in degrees to be
        #                             tolerated in the symmetry finding.
        #
        #     :return: The space group number
        #     :rtype : (int)
        #     """
        #     return int(self.spacegroup(symprec, angle_tolerance).split()[-1].strip()[1:-1])

        # def refine_cell(self, symprec=1e-5, angle_tolerance=-1.0):
        #     from pychemia import Structure
        #     natom = self.structure.natom
        #     cell = np.array(self.structure.cell.transpose(), dtype='double', order='C')
        #
        #     pos = np.zeros((natom * 4, 3), dtype='double')
        #     pos[:natom] = self._reduced
        #
        #     numbers = np.zeros(natom * 4, dtype='intc')
        #     numbers[:natom] = np.array(self._numbers, dtype='intc')
        #
        #     natom_bravais = spg.spglib.spg.refine_cell(cell, pos, numbers, natom, symprec, angle_tolerance)
        #     if natom_bravais == 0:
        #         return self.structure.copy()
        #
        #     cell = np.array(cell.T, dtype='double', order='C')
        #     reduced = np.array(pos[:natom_bravais], dtype='double', order='C')
        #     symbols = [self.structure.species[x] for x in (numbers[:natom_bravais] - 1)]
        #
        #     return Structure(cell=cell, symbols=symbols, reduced=reduced)
        #
        # def find_primitive(self, symprec=1e-5, angle_tolerance=1.0):
        #     """
        #     Search the primitive pychemia Structure using the tolerances.
        #     If no primitive cell is found a copy of the original structure is returned
        #
        #     :param symprec: (float) Tolerance of distance between atomic positions and between lengths of
        #                             lattice vectors to be tolerated in the symmetry finding.
        #     :param angle_tolerance: (float) Tolerance of angle between lattice vectors in degrees to
        #                                     be tolerated in the symmetry finding.
        #
        #     :return: A new pychemia Structure in a Bravais lattice
        #     :rtype : (pychemia.Structure)
        #     """
        #     from pychemia import Structure
        #     # Create copies of the arguments
        #     cell = np.array(self._transposed_cell, dtype='double', order='C')
        #     reduced = np.array(self._reduced, dtype='double', order='C')
        #     numbers = np.array(self._numbers, dtype='intc')
        #
        #     natom_prim = spg.spglib.spg.primitive(cell, reduced, numbers, symprec, angle_tolerance)
        #     symbols = [self.structure.species[x] for x in (numbers[:natom_prim] - 1)]
        #     reduced = reduced[:natom_prim]
        #
        #     if natom_prim > 0:
        #         return Structure(cell=cell.T, reduced=reduced, symbols=symbols)
        #     else:
        #         return self.structure.copy()
        #



import pychemia
import unittest
import numpy as np


class CrystalSymmetryTest(unittest.TestCase):
    def test_optimized_grid(self):
        """
        Tests (pychemia.crystal.symmetry)                            :
        """
        from pychemia import pcm_log
        pcm_log.debug("CrystalSymmetryTest")
        st = pychemia.code.vasp.read_poscar('pychemia/test/data/SbBi/POSCAR')
        cs = pychemia.crystal.CrystalSymmetry(st)
        assert cs.number() == 160
        assert cs.symbol() == u'R3m'
        pr = cs.find_primitive()
        assert np.abs(st.volume - pr.volume) < 1E-6
        assert cs.crystal_system() == u'Trigonal'
        ss = cs.symmetrize()
        assert np.abs(st.volume - ss.volume) < 1E-6



from .searcher import Searcher


class SimulatedAnnealing(Searcher):
    def __init__(self, population, params, generation_size=32, stabilization_limit=10):
        self.population = population
        Searcher.__init__(self, self.population, generation_size, stabilization_limit)
        self.params = None
        self.set_params(params)

    def set_params(self, params):
        self.params = params

    def run_one(self):
        pass

    def get_params(self):
        pass



#!/usr/bin/env python

import getopt
import os
import random
import signal
import socket
import subprocess
import sys
import time

from pychemia.utils.computing import get_int

__author__ = 'Guillermo Avendano-Franco'


class TimedOutExc(Exception):
    pass


def deadline(timeout, *args):
    def decorate(f):
        def handler(signum, frame):
            print('signum:', signum)
            print('frame:', frame)
            raise TimedOutExc()

        def new_f(*args):
            signal.signal(signal.SIGALRM, handler)
            signal.alarm(timeout)
            return f(*args)

        new_f.__name__ = f.__name__
        return new_f

    return decorate


def usage(name):
    print("""
NAME
    %s

DESCRIPTION
    PyChemia Client intended to be run by compute nodes


OPTIONS

    --help, -h
        Return information on the options and use of this script

    --port, -p <int> (Default: 10000)
        The UDP port that will be used for transfer data with clients

    --workdir, w <string>
        Working directory were files will be created and jobs runned

    --ip, -i <string>
        IP address to bind the server

""" % os.path.basename(name))


@deadline(60)
def inquirer(ip, port):
    # Create a UDP socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    server_address = (ip, port)
    print('Connecting to %s:%d' % (ip, port), file=sys.stderr)

    try:
        message = 'COUNT'
        # Send data
        print('sending "%s"' % message, file=sys.stderr)
        sock.sendto(message, server_address)

        # Receive response
        print('waiting to receive', file=sys.stderr)
        data, server = sock.recvfrom(4096)
        print('received "%s"' % data, file=sys.stderr)

        message = 'WORKDIR'
        # Send data
        print('sending "%s"' % message, file=sys.stderr)
        sock.sendto(message, server_address)

        # Receive response
        print('waiting to receive', file=sys.stderr)
        data, server = sock.recvfrom(4096)
        print('received "%s"' % data, file=sys.stderr)
        workdir = data

        message = 'GET'
        # Send data
        print('sending "%s"' % message, file=sys.stderr)
        sock.sendto(message, server_address)

        # Receive response
        print('waiting to receive', file=sys.stderr)
        data, server = sock.recvfrom(4096)
        print('received "%s"' % data, file=sys.stderr)
        entry_id = data

    finally:
        print('closing socket', file=sys.stderr)
        sock.close()

    return workdir + os.sep + entry_id, entry_id


def executor(workdir):
    print('Work directory', workdir)
    os.chdir(workdir)
    subprocess.call(['python', 'executor.py'])


@deadline(60)
def finisher(entry_id, ip, port):
    # Create a UDP socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    server_address = (ip, port)

    try:
        message = 'FINISHED:' + str(entry_id)
        # Send data
        print('sending "%s"' % message, file=sys.stderr)
        sock.sendto(message, server_address)

        # Receive response
        print('waiting to receive', file=sys.stderr)
        data, server = sock.recvfrom(4096)
        print('received "%s"' % data, file=sys.stderr)

    finally:
        print('closing socket', file=sys.stderr)
        sock.close()


def main(argv):
    try:
        opts, args = getopt.getopt(argv[1:], "hi:p:", ["help", "ip=", "port="])
    except getopt.GetoptError:
        print('Wrong parsing of options')
        usage(argv[0])
        sys.exit(2)

    # Default Values
    port = None
    ip = None

    for opt, arg in opts:

        if opt in ("-h", "--help"):
            usage(argv[0])
            sys.exit()
        elif opt in ("-p", "--port"):
            port = get_int(arg)
        elif opt in ("-i", "--ip"):
            ip = arg

    if ip is None:
        ip = socket.gethostbyname(socket.gethostname())

    print('PyChemia Client connecting to server on : ', ip)

    workdir = None
    entry_id = None
    while True:
        if port is not None:
            try:
                workdir, entry_id = inquirer(ip, port)
            except TimedOutExc as e:
                print("INQUIRER took too long: %s", e.message)
        else:
            lt = time.localtime()
            random.seed(lt.tm_yday * 24 + lt.tm_hour)
            rndport = random.randint(10000, 20000)
            print('PyChemia Client using port : ', rndport)
            try:
                workdir, entry_id = inquirer(ip, rndport)
            except TimedOutExc as e:
                print("Error({0}): {1}".format(e, e.message))

        print('We got the workdir: ', workdir)
        print('The entry_id is : ', entry_id)

        if workdir is not None and entry_id is not None:
            if entry_id == '':
                print('No entries to execute, taking a nap')
                time.sleep(15)
            else:
                break
        else:
            'Inquierer failed to get data, retry...'

    executor(workdir)

    if port is not None:
        finisher(entry_id, ip, port)
    else:
        lt = time.localtime()
        random.seed(lt.tm_yday * 24 + lt.tm_hour)
        rndport = random.randint(10000, 20000)
        print('PyChemia Client using port : ', rndport)

        try:
            finisher(entry_id, ip, rndport)
        except TimedOutExc as e:
            print("Error({0}): {1}".format(e, e.message))


if __name__ == "__main__":
    main(sys.argv)



import os
import re
import subprocess
import numpy as np

__author__ = 'Guillermo Avendano Franco'


def get_point_group(st, executable='symmol', dcm=0.2, dcme=0.2):

    if not cmd_exists(executable):
        return 'no symmol'

    wf = open('symmol.in', 'w')
    wf.write(' %9.5f %9.5f %9.5f %9.5f %9.5f %9.5f\n' % (1, 1, 1, 90, 90, 90))
    wf.write(' %d %d %9.6f %9.6f\n' % (1, 1, dcm, dcme))
    for i in st:
        name = i.symbols[0]
        wf.write('%6s%2d%9.5f%9.5f%9.5f\n' % tuple([name.ljust(6), 1] + list(i.position)))
    wf.close()

    rf = open('symmol.in')

    if np.max(np.abs(st.positions.flatten())) >= 100:
        return 'ERR'

    data = subprocess.check_output([executable], stdin=rf)
    data = data.decode('utf-8')
    ans = re.findall('Schoenflies symbol =([\d\w\s]+)CSM', data)
    rf.close()
    if len(ans) != 1:
        pg = 'no symmetry'
    else:
        pg = ans[0].strip()
        os.remove('symmol.in')
    return pg


def cmd_exists(cmd):
    return subprocess.call("type " + cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0



import os
import re
import subprocess

import numpy as np

from pychemia import pcm_log, Structure
from pychemia.utils.serializer import generic_serializer
from pychemia.utils.periodic import atomic_number, atomic_symbol
from ..codes import Codes


class FireBall(Codes):
    def __init__(self, fdata_path=None):
        self.workdir = None
        # The five sections of Fireball
        self.option = {}
        self.tds = {}
        self.quench = {}
        self.tds = {}
        self.mesh = {}
        self.output = {}

        self.fdata_path = os.path.abspath(fdata_path)
        self.structure = None
        self.binary = 'fireball.x'
        self.runner = None
        self.kpoints = None
        Codes.__init__(self)

    @property
    def to_dict(self):
        ret = {}
        if self.option:
            ret['option'] = self.option
        if self.output:
            ret['output'] = self.output
        if self.quench:
            ret['quench'] = self.quench
        if self.mesh:
            ret['mesh'] = self.mesh
        if self.tds:
            ret['tds'] = self.tds
        return ret

    @staticmethod
    def from_dict(dictionary):
        ret = FireBall()
        if 'option' in dictionary:
            ret.option = dictionary['option']
        if 'output' in dictionary:
            ret.option = dictionary['output']
        if 'quench' in dictionary:
            ret.option = dictionary['quench']
        if 'mesh' in dictionary:
            ret.option = dictionary['mesh']
        if 'tds' in dictionary:
            ret.option = dictionary['tds']
        return ret

    def initialize(self, structure, workdir=None, kpoints=None, binary='fireball.x'):
        assert structure.is_perfect
        self.structure = structure
        if workdir is not None:
            self.workdir = workdir
        else:
            self.workdir = '.'
        if not os.path.lexists(self.workdir):
            os.mkdir(self.workdir)
        self.kpoints = kpoints
        self.binary = binary

    def set_inputs(self, rms=0.1):
        self.write_input(filename=self.workdir + os.sep + 'fireball.in')
        self.write_basis(filename=self.workdir + os.sep + 'input.bas')
        if self.structure.is_periodic:
            self.write_lattice(filename=self.workdir + os.sep + 'input.lvs')
            self.write_kpoints(filename=self.workdir + os.sep + 'input.kpts')
        self.link_fdata()
        wf = open(self.workdir + os.sep + 'rms.input', 'w')
        wf.write('%f\n' % rms)
        wf.close()

    def get_outputs(self):
        pass

    def run(self, use_mpi=False, omp_max_threads=0, mpi_num_procs=1):
        cwd = os.getcwd()
        os.chdir(self.workdir)
        stdout = open('fireball.log', 'w')
        sp = subprocess.Popen(self.binary, stdout=stdout)
        os.chdir(cwd)
        self.runner = sp
        return sp

    def run_status(self):
        if self.runner is None:
            pcm_log.info('Fireball not finish')
            filename = self.workdir + os.sep + 'fireball.log'
            if os.path.exists(filename):
                read_fireball_stdout(filename=filename)
            return
        if self.runner.poll() == 0:
            pcm_log.info('Fireball complete normally')

    def finalize(self):
        pass

    def write_input(self, filename='fireball.in'):

        wf = open(filename, 'w')
        wf.write(str(self))
        wf.close()

    def __str__(self):
        ret = ''
        if self.option:
            ret += '&OPTION\n'
            for variable in sorted(self.option):
                if isinstance(self.option[variable], str):
                    ret += "%s = '%s'\n" % (variable, str(self.option[variable]))
                else:
                    ret += '%s = %s\n' % (variable, str(self.option[variable]))
            ret += '&END\n'

        if self.output:
            ret += '&OUTPUT\n'
            for variable in sorted(self.output):
                ret += '%s = %s\n' % (variable, str(self.output[variable]))
            ret += '&END\n'

        if self.quench:
            ret += '&QUENCH\n'
            for variable in sorted(self.quench):
                ret += '%s = %s\n' % (variable, str(self.quench[variable]))
            ret += '&END\n'

        if self.mesh:
            ret += '&MESH\n'
            for variable in sorted(self.mesh):
                ret += '%s = %s\n' % (variable, str(self.mesh[variable]))
            ret += '&END\n'

        if self.tds:
            ret += '&TDS\n'
            for variable in sorted(self.tds):
                ret += '%s = %s\n' % (variable, str(self.tds[variable]))
            ret += '&END\n'

        return ret

    def write_basis(self, filename='input.bas'):
        write_geometry_bas(self.structure, filename)

    def write_lattice(self, filename='input.lvs'):
        wf = open(filename, 'w')
        for i in range(3):
            wf.write('%13.7f %13.7f %13.7f\n' % (self.structure.cell[i, 0],
                                                 self.structure.cell[i, 1],
                                                 self.structure.cell[i, 2]))
        wf.close()

    def write_kpoints(self, filename):
        pass

    def link_fdata(self):
        if self.fdata_path is not None:
            linkpath = self.workdir + os.sep + 'Fdata'
            if os.path.exists(linkpath):
                os.remove(linkpath)
            os.symlink(self.fdata_path, linkpath)

    def cluster_relaxation(self):
        self.option = {'basisfile': 'input.bas', 'nstepf': 5000, 'iquench': -3, 'icluster': 1, 'iqout': 3, 'dt': 0.25}
        self.output = {'iwrtxyz': 1}


def read_fireball_stdout(filename):
    rf = open(filename, 'r')
    data = rf.read()

    atom_data = re.findall(r'Atom Coordinates from Basis File:([\d\w\s=\-.#]+)===\n', data)[0]

    symbols = []
    positions = []
    for iline in atom_data.split('\n'):
        # print iline
        tmp = iline.split()
        if len(tmp) == 6 and tmp[0].isdigit():
            symbols.append(tmp[1])
            positions.append([float(x) for x in tmp[2:5]])

    natom = len(symbols)
    initial_positions = np.array(positions)
    # print symbols
    # print initial_positions

    forces_data = re.findall(r'The grand total force \(eV/A\):([\w\d\s.\-=+]*)Cartesian', data)
    # print len(forces_data)

    forces = np.zeros((len(forces_data), natom, 3))
    # print forces.shape

    iteration = 0
    for idata in forces_data:
        for iline in idata.split('\n'):
            if 'iatom' in iline:
                fields = iline.split()
                forces[iteration, int(fields[2]) - 1] = np.array(fields[-3:], dtype=float)
        iteration += 1

    ret = re.findall('Cartesian Forces:\s*Max = ([=\w\s\d.]*)RMS = ([\s\d.]*)\n', data)
    ret = [[float(x) for x in y] for y in ret]
    max_force = [x[0] for x in ret]
    rms = [x[1] for x in ret]

    energy_data = re.findall(r'---------- T H E  T O T A L  E N E R G Y -----------([\s\w\d.\-=/]+)--- \n', data)

    energy = []
    for idata in energy_data:
        for iline in idata.split('\n'):
            if 'Time step' in iline:
                tmp = iline.split()
                ienergy = {'Time_step': int(tmp[3]), 'SCF_step': int(tmp[7]), 'etot/atom': float(tmp[10])}
            elif len(iline.split('=')) == 2:
                tmp = iline.split('=')
                ienergy[tmp[0].strip()] = float(tmp[1])
        energy.append(ienergy)

    gt_data = re.findall(r'Grand Total = nuclear kinetic \+ potential = [\s\-\d.]*', data)
    gt = [float(i.split()[-1]) for i in gt_data]

    gtpa_data = re.findall(r'grand total energy per atom = [\s\-\d.]*', data)
    gtpa = [float(i.split()[-1]) for i in gtpa_data]

    ret = {'symbols': symbols,
           'initial_positions': initial_positions,
           'forces': generic_serializer(forces),
           'energetics': energy,
           'max_force': max_force,
           'rms_force': rms,
           'grand_total': gt,
           'grand_total_energy_per_atom': gtpa
           }
    return ret


def read_geometry_bas(filename):
    rf = open(filename, 'r')

    natom = int(rf.readline())
    symbols = []
    positions = np.zeros((natom, 3))

    for i in range(natom):
        line = rf.readline().split()
        symbols.append(atomic_symbol(int(line[0])))
        positions[i] = np.array(line[1:])

    return Structure(symbols=symbols, positions=positions, periodicity=False)


def write_geometry_bas(structure, filename):
    wf = open(filename, 'w')
    wf.write('%3d\n' % structure.natom)
    for i in range(structure.natom):
        if structure.is_periodic:
            x = structure.reduced[i, 0]
            y = structure.reduced[i, 1]
            z = structure.reduced[i, 2]
        else:
            x = structure.positions[i, 0]
            y = structure.positions[i, 1]
            z = structure.positions[i, 2]
        wf.write('%3d %13.7f %13.7f %13.7f\n' % (atomic_number(structure.symbols[i]), x, y, z))
    wf.close()


def get_fdata_info(fdata_path='Fdata'):
    rf = open(fdata_path + os.sep + 'info.dat')
    data = rf.read()
    ret = re.findall('([-. \d\w]*) - ([ \d\w;]*) \n', data)
    res = {}
    for i in ret:
        key = i[1]
        value = i[0]
        if key == 'Information for this species':
            cur_atom = int(value)
            res[cur_atom] = {}
        elif key == 'Element':
            res[cur_atom]['Element'] = value.strip()
        elif key == 'Nuclear Z':
            res[cur_atom]['Nuclear Z'] = int(value)
        elif key == 'Atomic Mass':
            res[cur_atom]['Atomic Mass'] = float(value)
        elif key == 'Number of shells; L for each shell':
            res[cur_atom]['Number of shells; L for each shell'] = int(value)
        elif key == 'Radial cutoffs PP':
            res[cur_atom]['Radial cutoffs PP'] = float(value)
        elif key == 'Atomic energy':
            res[cur_atom]['Atomic energy'] = float(value)
    return res


def read_fireball_in(fpath='fireball.in'):

    rf = open(fpath)
    data = rf.readlines()
    ret = {}

    for iline in data:
        if iline.startswith('&'):
            if iline.startswith('&END'):
                curkey = None
            else:
                curkey = iline[1:].strip()
        elif '=' in iline:
            varname = iline.split('=')[0].strip()
            value = iline.split('=')[1].strip()
            if curkey is not None and curkey not in ret:
                ret[curkey] = {}
            ret[curkey][varname] = convert_value(value)
    return ret


def read_eigen(fpath='eigen.dat'):

    rf = open(fpath)
    data = rf.read()
    # Number of Eigenvalues
    nval = int(data.split()[1])
    # The eigenvalues
    eigen = [float(x) for x in data.split()[7:]]
    assert(len(eigen) == nval)
    return eigen


def read_final_fireball_relax(fpath):

    output = read_fireball_stdout(fpath)
    ret = {'energetics': output['energetics'][-1],
           'forces': output['forces'][-1],
           'rms_force': output['rms_force'][-1],
           'max_force': output['max_force'][-1],
           'grand_total': output['grand_total'][-1],
           'grand_total_energy_per_atom': output['grand_total_energy_per_atom'][-1]}

    return ret


def convert_value(value):

    ret = None
    try:
        ret = int(value)
    except ValueError:
        try:
            ret = float(value)
        except ValueError:
            try:
                ret = [int(x) for x in value.split()]
            except ValueError:
                ret = value
    return ret


def read_param(fpath='param.dat'):
    rf = open(fpath)
    data = rf.readlines()
    ret = {}
    for line in data:
        if ':' in line:
            key = line.split(':')[0].strip()
            value = line.split(':')[1].strip()
            if value == '':
                curkey = key
            else:
                if curkey not in ret:
                    ret[curkey] = {}
                ret[curkey][key] = convert_value(value)
        else:
            if '=' not in line and len(line.strip()) > 0:
                curkey = line.strip()
    return ret


def read_lvs(fpath='input.lvs'):
    cell = np.zeros((3, 3))
    rf = open(fpath)
    data = rf.readlines()
    for i in range(len(data)):
        if len(data[i].split()) > 2:
            cell[i] = [float(x) for x in data[i].split()[:3]]
    return list(cell.flatten())
